@@ -1,21 +1,30 @@
 /*-------------------------------------------------------------------------
-   Register Declarations for the SiLabs C8051F36x Processor Range
+   C8051F360.h - Register Declarations for the SiLabs C8051F36x
+   Processor Range
 
-   Copyright (C) 2007 - Maarten Brock, sourceforge.brock@dse.nl
+   Copyright (C) 2007, Maarten Brock, sourceforge.brock@dse.nl
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 2.1 of the License, or (at your option) any later version.
+   This library is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 2.1, or (at your option) any
+   later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+   GNU General Public License for more details.
 
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+   You should have received a copy of the GNU General Public License 
+   along with this library; see the file COPYING. If not, write to the
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301, USA.
+
+   As a special exception, if you link this library with other files,
+   some of which are compiled with SDCC, to produce an executable,
+   this library does not by itself cause the resulting executable to
+   be covered by the GNU General Public License. This exception does
+   not however invalidate any other reasons why the executable file
+   might be covered by the GNU General Public License.
 -------------------------------------------------------------------------*/
 
 #ifndef C8051F360_H
@@ -103,7 +112,7 @@ SFR(P2,         0xA0);      // Port 2
 SFR(SPI0CFG,    0xA1);      // SPI Configuration
 SFR(SPI0CKR,    0xA2);      // SPI Clock Rate Control
 SFR(SPI0DAT,    0xA3);      // SPI Data
-SFR(SFRNEXT,    0xA7);      // SFR Page Select
+SFR(SFRPAGE,    0xA7);      // SFR Page Select
 SFR(IE,         0xA8);      // Interrupt Enable
    SBIT(EX0,      0xA8, 0); // Enable External Interrupt 0
    SBIT(ET0,      0xA8, 1); // Enable Timer 0 Interrupt
@@ -164,7 +173,7 @@ SFR(TMR2CN,     0xC8);      // Timer/Counter 2 Control
    SBIT(TF2CEN,   0xC8, 4); // Timer 2 Low-Frequency Oscillator Capture Enable
    SBIT(TF2LEN,   0xC8, 5); // Timer 2 Low Byte Interrupt Enable
    SBIT(TF2L,     0xC8, 6); // Timer 2 Low Byte Overflow Flag
-   SBIT(TF2L,     0xC8, 7); // Timer 2 High Byte Overflow Flag
+   SBIT(TF2H,     0xC8, 7); // Timer 2 High Byte Overflow Flag
 SFR16(TMR2RL,   0xCA);      // Timer/Counter 2 Reload Word
    SFR(TMR2RLL,   0xCA);    // Timer/Counter 2 Reload Low Byte
    SFR(TMR2RLH,   0xCB);    // Timer/Counter 2 Reload High Byte
@@ -239,9 +248,9 @@ SFR16(PCA0,     0xF9);      // PCA Counter Word
 SFR16(PCA0CP0,  0xFB);      // PCA Capture 0 Word
    SFR(PCA0CPL0,  0xFB);    // PCA Capture 0 Low Byte
    SFR(PCA0CPH0,  0xFC);    // PCA Capture 0 High Byte
-SFR16(PCA0CP5,  0xFD);      // PCA Capture 5 Word
-   SFR(PCA0CPL5,  0xFD);    // PCA Capture 5 Low Byte
-   SFR(PCA0CPH5,  0xFE);    // PCA Capture 5 High Byte
+SFR16(PCA0CP4,  0xFD);      // PCA Capture 4 Word
+   SFR(PCA0CPL4,  0xFD);    // PCA Capture 4 Low Byte
+   SFR(PCA0CPH4,  0xFE);    // PCA Capture 4 High Byte
 SFR(VDM0CN,     0xFF);      // VDD Monitor Control
 
 /*  Page 0x00 */
