@@ -1,20 +1,30 @@
-/* Fast printf routine for use with sdcc/mcs51
- * Copyright (c) 2004, Paul Stoffregen, paul@pjrc.com
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- */
+/*-------------------------------------------------------------------------
+   printf_fast.c - Fast printf routine for use with sdcc/mcs51
+
+   Copyright (C) 2004, Paul Stoffregen, paul@pjrc.com
+
+   This library is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 2.1, or (at your option) any
+   later version.
+
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License 
+   along with this library; see the file COPYING. If not, write to the
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301, USA.
+
+   As a special exception, if you link this library with other files,
+   some of which are compiled with SDCC, to produce an executable,
+   this library does not by itself cause the resulting executable to
+   be covered by the GNU General Public License. This exception does
+   not however invalidate any other reasons why the executable file
+   might be covered by the GNU General Public License.
+-------------------------------------------------------------------------*/
 
 /******************************************************************/
 /**                                                              **/
@@ -133,7 +143,7 @@ static __data unsigned int i2bcd_tmp;  // slow 32 int conversion needs temp spac
 #else // defines are compatible with printf_fast
 
 
-void PRINTF_FAST(__code char *fmt, ...) __reentrant
+void PRINTF_FAST(__code const char *fmt, ...) __reentrant
 {
 	fmt;	/* suppress unreferenced variable warning */
 
@@ -414,6 +424,11 @@ printf_uint_ck8:
 	/* print the digit zero and skip all the hard work! */
 	mov	a, r1
 	jnz	printf_uint_begin
+#ifdef FLOAT
+	/* never use the "just print zero" shortcut if we're printing */
+	/* the integer part of a float  (fixes bug 1255403)  */
+	jb	_continue_float, printf_uint_begin
+#endif
 #ifdef FIELD_WIDTH
 	jnb	_field_width_flag, printf_uint_zero
 	mov	a, _field_width
@@ -425,14 +440,8 @@ printf_uint_zero:
 	//mov	a, #'0'
 	mov	a, #48
 	lcall	printf_putchar
-#ifdef FLOAT
-	jnb	_continue_float, 0001$
-	ret
-0001$:
-#endif
 	ljmp	printf_main_loop
 
-
 printf_uint_begin:
 	push	dpl
 	push	dph
@@ -486,7 +495,7 @@ printf_uifw_8:
 	jnz	printf_uifw_sub
 	dec	r1
 printf_uifw_sub:
-	;r1 has the number of digits for the number
+	//r1 has the number of digits for the number
 	mov	a, _field_width
 	mov	c, _negative_flag
 	subb	a, r1
@@ -766,8 +775,12 @@ print_float_default_width:
 	// default fractional field width (between 0 to 7)
 	// attempt to scale the default number of fractional digits
 	// based on the magnitude of the float
-	mov	ar1, r0		// r0 points to first byte of float
-	dec	r1		// r1 points to second byte of float
+	mov	a, @r0
+	anl	a, #0x7F	// ignore sign bit
+	mov	r2, a		// r2 is first byte of float
+	dec	r0
+	mov	ar3, @r0	// r3 is second byte of float
+	inc	r0
 	mov	r6, dpl
 	mov	r7, dph
 	mov	dptr, #_float_range_table
@@ -775,11 +788,11 @@ print_float_default_width:
 print_float_default_loop:
 	clr	a
 	movc	a, @a+dptr
-	add	a, @r1
+	add	a, r3
 	inc	dptr
 	clr	a
 	movc	a, @a+dptr
-	addc	a, @r0
+	addc	a, r2
 	jnc	print_float_default_done
 	inc	dptr
 	djnz	r5, print_float_default_loop
