@@ -347,46 +347,56 @@ DEFSETFUNC (findCheaperOp)
   /* and this is not the defining one          */
   if (cop->key == cdp->key)
     {
-
       /* do a special check this will help in */
       /* constant propagation & dead code elim */
       /* for assignments only                 */
-      if (cdp->diCode->op == '=') {
-        /* if the result is volatile then return result */
-        if (IS_OP_VOLATILE (IC_RESULT (cdp->diCode)))
-          *opp = IC_RESULT (cdp->diCode);
-        else
-          /* if this is a straight assignment and
-             left is a temp then prefer the temporary to the
-             true symbol */
-          if (!POINTER_SET (cdp->diCode) &&
-              IS_ITEMP (IC_RESULT (cdp->diCode)) &&
-              IS_TRUE_SYMOP (IC_RIGHT (cdp->diCode)))
-            *opp = IC_RESULT (cdp->diCode);
-          else {
-            /* if straight assignment and both
-               are temps then prefer the one that
-               will not need extra space to spil, also
-               take into consideration if right side
-               is an induction variable
-            */
-            if (!POINTER_SET (cdp->diCode) &&
-                IS_ITEMP (IC_RESULT (cdp->diCode)) &&
-                IS_ITEMP (IC_RIGHT (cdp->diCode)) &&
-                !OP_SYMBOL (IC_RIGHT (cdp->diCode))->isind &&
-                !OP_SYMBOL(IC_RIGHT (cdp->diCode))->isreqv &&
-                ((!SPIL_LOC (IC_RIGHT (cdp->diCode)) &&
-                  SPIL_LOC (IC_RESULT (cdp->diCode))) ||
-                 (SPIL_LOC (IC_RESULT (cdp->diCode)) &&
-                  SPIL_LOC (IC_RESULT (cdp->diCode)) ==
-                  SPIL_LOC (IC_RIGHT (cdp->diCode)))))
+      if (cdp->diCode->op == '=')
+        {
+          /* if the result is volatile then return result */
+          if (IS_OP_VOLATILE (IC_RESULT (cdp->diCode)))
+            {
               *opp = IC_RESULT (cdp->diCode);
-            else
-              *opp = IC_RIGHT (cdp->diCode);
-          }
-      }
+            }
+          else
+            {
+              /* if this is a straight assignment and
+                 left is a temp then prefer the temporary to the
+                 true symbol */
+              if (!POINTER_SET (cdp->diCode) &&
+                  IS_ITEMP (IC_RESULT (cdp->diCode)) &&
+                  IS_TRUE_SYMOP (IC_RIGHT (cdp->diCode)))
+                {
+                  *opp = IC_RESULT (cdp->diCode);
+                }
+              else
+                {
+                  /* if straight assignment and both
+                     are temps then prefer the one that
+                     will not need extra space to spil, also
+                     take into consideration if right side
+                     is an induction variable
+                  */
+                  if (!POINTER_SET (cdp->diCode) &&
+                      IS_ITEMP (IC_RESULT (cdp->diCode)) &&
+                      IS_ITEMP (IC_RIGHT (cdp->diCode)) &&
+                      !OP_SYMBOL (IC_RIGHT (cdp->diCode))->isind &&
+                      !OP_SYMBOL(IC_RIGHT (cdp->diCode))->isreqv &&
+                      ((!SPIL_LOC (IC_RIGHT (cdp->diCode)) && SPIL_LOC (IC_RESULT (cdp->diCode))) ||
+                       (SPIL_LOC (IC_RESULT (cdp->diCode)) && SPIL_LOC (IC_RESULT (cdp->diCode)) == SPIL_LOC (IC_RIGHT (cdp->diCode)))))
+                    {
+                      *opp = IC_RESULT (cdp->diCode);
+                    }
+                  else
+                    {
+                      *opp = IC_RIGHT (cdp->diCode);
+                    }
+                }
+            }
+        }
       else
-        *opp = IC_RESULT (cdp->diCode);
+        {
+          *opp = IC_RESULT (cdp->diCode);
+        }
     }
 
   /* if this is an assign to a temp. then check
@@ -397,7 +407,9 @@ DEFSETFUNC (findCheaperOp)
       cop->key == IC_RIGHT (cdp->diCode)->key &&
       !isGlobalInNearSpace (IC_RIGHT (cdp->diCode)) &&
       IS_ITEMP (IC_RESULT (cdp->diCode)))
-    *opp = IC_RESULT (cdp->diCode);
+    {
+      *opp = IC_RESULT (cdp->diCode);
+    }
 
   if ((*opp) &&
       (isOperandLiteral(*opp) || !checkSign ||
@@ -405,12 +417,9 @@ DEFSETFUNC (findCheaperOp)
         IS_SPEC(operandType (cop)) && IS_SPEC(operandType (*opp)) &&
         (SPEC_USIGN(operandType (cop))==SPEC_USIGN(operandType (*opp)) &&
          (SPEC_LONG(operandType (cop))==SPEC_LONG(operandType (*opp)))))))
-      {
-
-      if ((isGlobalInNearSpace (cop) &&
-           !isOperandLiteral (*opp)) ||
-          isOperandVolatile (*opp, FALSE)
-        )
+    {
+      if ((isGlobalInNearSpace (cop) && !isOperandLiteral (*opp)) ||
+          isOperandVolatile (*opp, FALSE))
         {
           *opp = NULL;
           return 0;
@@ -433,33 +442,30 @@ DEFSETFUNC (findCheaperOp)
           && isOperandLiteral(*opp)
           && SPEC_NOUN(operandType(*opp)) == SPEC_NOUN(operandType(cop))
           && SPEC_USIGN(operandType(*opp)) != SPEC_USIGN(operandType(cop)))
-      {
+        {
           SPEC_USIGN(operandType(*opp)) = SPEC_USIGN(operandType(cop));
-      }
+        }
 
       if (IS_SPEC(operandType (cop)) && IS_SPEC(operandType (*opp)) &&
           SPEC_NOUN(operandType(cop)) != SPEC_NOUN(operandType(*opp)))
         {
-            // special case: we can make an unsigned char literal
-            // into an int literal with no cost.
-            if (isOperandLiteral(*opp)
-             && SPEC_NOUN(operandType(*opp)) == V_CHAR
-             && SPEC_NOUN(operandType(cop)) == V_INT)
+          // special case: we can make an unsigned char literal
+          // into an int literal with no cost.
+          if (isOperandLiteral(*opp) &&
+              SPEC_NOUN(operandType(*opp)) == V_CHAR &&
+              SPEC_NOUN(operandType(cop)) == V_INT)
             {
-                *opp = operandFromOperand (*opp);
-                SPEC_NOUN(operandType(*opp)) = V_INT;
+              *opp = operandFromOperand (*opp);
+              SPEC_NOUN(operandType(*opp)) = V_INT;
             }
-            else
+          else
             {
-                // No clue...
-                *opp = NULL;
-                return 0;
+              // No clue...
+              *opp = NULL;
+              return 0;
             }
-
         }
-
       return 1;
-
     }
   *opp=NULL;
   return 0;
@@ -778,7 +784,6 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
       IS_OP_LITERAL (IC_LEFT (ic)) &&
       IS_OP_LITERAL (IC_RIGHT (ic)))
     {
-
       IC_RIGHT (ic) = operandOperation (IC_LEFT (ic),
                                         IC_RIGHT (ic),
                                         ic->op,
@@ -787,7 +792,6 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
       IC_LEFT (ic) = NULL;
       SET_RESULT_RIGHT (ic);
       return;
-
     }
   /* if not ifx & only one operand present */
   if (IC_RESULT (ic) &&
@@ -795,7 +799,6 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
       IS_OP_LITERAL (IC_LEFT (ic)) &&
       !IC_RIGHT (ic))
     {
-
       IC_RIGHT (ic) = operandOperation (IC_LEFT (ic),
                                         IC_RIGHT (ic),
                                         ic->op,
@@ -806,16 +809,13 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
       return;
     }
 
-
   /* a special case : or in short a kludgy solution will think
      about a better solution over a glass of wine someday */
   if (ic->op == GET_VALUE_AT_ADDRESS)
     {
-
       if (IS_ITEMP (IC_RESULT (ic)) &&
           IS_TRUE_SYMOP (IC_LEFT (ic)))
         {
-
           ic->op = '=';
           IC_RIGHT (ic) = operandFromOperand (IC_LEFT (ic));
           IC_RIGHT (ic)->isaddr = 0;
@@ -840,10 +840,8 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
           IC_LEFT (ic) = NULL;
           return;
         }
-
     }
 
-
   /* depending on the operation */
   switch (ic->op)
     {
@@ -957,7 +955,7 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
     case '*':
       if (IS_OP_LITERAL (IC_LEFT (ic)))
         {
-         double leftValue = operandLitValue (IC_LEFT (ic));
+          double leftValue = operandLitValue (IC_LEFT (ic));
 
           if (leftValue == 0.0)
             {
@@ -1038,6 +1036,25 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
             }
           if (rightValue == -1.0)
             {
+              /* '*' can have two unsigned chars as operands */
+              /* and an unsigned int as result.              */
+              if (IS_INTEGRAL (operandType (IC_LEFT (ic))))
+                {
+                  if ((getSize (operandType (IC_LEFT (ic))) < (unsigned int) INTSIZE) &&
+                      (getSize (operandType (IC_LEFT (ic))) < getSize (operandType (IC_RESULT (ic)))))
+                    {
+                      operand * op;
+                      iCode * newic;
+                      /* Widen to int. */
+                      op = operandFromOperand (IC_RESULT (ic));
+                      op->type = TYPE;
+                      setOperandType (op, INTTYPE);
+                      newic = newiCode (CAST, op, IC_LEFT (ic));
+                      IC_RESULT (newic) = newiTempOperand (INTTYPE, TRUE);
+                      addiCodeToeBBlock (ebp, newic, ic);
+                      IC_LEFT (ic) = IC_RESULT (newic);
+                    }
+                }
               /* convert x * -1 to -x */
               ic->op = UNARYMINUS;
               IC_RIGHT (ic) = NULL;
@@ -1054,20 +1071,58 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
           IC_LEFT (ic) = NULL;
           IC_RESULT (ic) = operandFromOperand (IC_RESULT (ic));
           IC_RESULT (ic)->isaddr = 0;
-          break;
+          return;
         }
-      /* if this is a division then check if right */
-      /* is one then change it to an assignment    */
-      if (IS_OP_LITERAL (IC_RIGHT (ic)) &&
-          operandLitValue (IC_RIGHT (ic)) == 1.0)
+      /* if this is a division then check if left is zero */
+      /* then change it to an assignment */
+      if (IS_OP_LITERAL (IC_LEFT (ic)) &&  (operandLitValue (IC_LEFT (ic)) == 0.0))
         {
-
           ic->op = '=';
           IC_RIGHT (ic) = IC_LEFT (ic);
           IC_LEFT (ic) = NULL;
           SET_RESULT_RIGHT (ic);
           return;
         }
+      /* if this is a division then check if right */
+      /* is one then change it to an assignment    */
+      if (IS_OP_LITERAL (IC_RIGHT (ic)))
+        {
+          double rightValue = operandLitValue (IC_RIGHT (ic));
+          if (rightValue == 1.0)
+            {
+              ic->op = '=';
+              IC_RIGHT (ic) = IC_LEFT (ic);
+              IC_LEFT (ic) = NULL;
+              SET_RESULT_RIGHT (ic);
+              return;
+            }
+          if (rightValue == -1.0)
+            {
+              /* '/' can have two unsigned chars as operands */
+              /* and an unsigned int as result.              */
+              if (IS_INTEGRAL (operandType (IC_LEFT (ic))))
+                {
+                  if ((getSize (operandType (IC_LEFT (ic))) < (unsigned int) INTSIZE) &&
+                      (getSize (operandType (IC_LEFT (ic))) < getSize (operandType (IC_RESULT (ic)))))
+                    {
+                      operand * op;
+                      iCode * newic;
+                      /* Widen to int. */
+                      op = operandFromOperand (IC_RESULT (ic));
+                      op->type = TYPE;
+                      setOperandType (op, INTTYPE);
+                      newic = newiCode (CAST, op, IC_LEFT (ic));
+                      IC_RESULT (newic) = newiTempOperand (INTTYPE, TRUE);
+                      addiCodeToeBBlock (ebp, newic, ic);
+                      IC_LEFT (ic) = IC_RESULT (newic);
+                    }
+                }
+              /* convert x / -1 to -x */
+              ic->op = UNARYMINUS;
+              IC_RIGHT (ic) = NULL;
+              return;
+            }
+        }
       break;
       /* if both are the same for an comparison operators */
     case EQ_OP:
@@ -1097,12 +1152,23 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
           sym_link *otype = operandType(IC_RIGHT(ic));
           sym_link *ctype = operandType(IC_LEFT(ic));
           /* if this is a cast of a literal value */
-          if (IS_OP_LITERAL (IC_RIGHT (ic)) &&
-              !(IS_GENPTR(ctype) && (IS_PTR(otype) && !IS_GENPTR(otype))))
+          if (IS_OP_LITERAL (IC_RIGHT (ic)))
             {
+              double litval = operandLitValue (IC_RIGHT (ic));
+              if (IS_GENPTR(ctype) && IS_PTR(otype))
+                {
+                  unsigned long gpVal = 0;
+                  const char *name = IS_SYMOP (IC_RESULT (ic)) ? OP_SYMBOL (IC_RESULT (ic))->name : NULL;
+                  if (IS_FUNCPTR(otype))
+                    gpVal = pointerTypeToGPByte (DCL_TYPE (otype->next), NULL, name);
+                  if (!IS_GENPTR(otype))
+                    gpVal = pointerTypeToGPByte (DCL_TYPE (otype), NULL, name);
+                  gpVal <<= ((GPTRSIZE - 1) * 8);
+                  gpVal |= (unsigned long)litval;
+                  litval = gpVal;
+                }
               ic->op = '=';
-              IC_RIGHT (ic) = operandFromValue (valCastLiteral (operandType (IC_LEFT (ic)),
-                                                                operandLitValue (IC_RIGHT (ic))));
+              IC_RIGHT (ic) = operandFromValue (valCastLiteral (operandType (IC_LEFT (ic)), litval));
               IC_LEFT (ic) = NULL;
               SET_ISADDR (IC_RESULT (ic), 0);
             }
@@ -1231,6 +1297,8 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
         }
       if (IS_OP_LITERAL (IC_RIGHT (ic)))
         {
+          unsigned val;
+
           /* if BITWISEOR then check if one of them is a zero */
           /* if yes turn it into assignment */
           if (operandLitValue (IC_RIGHT (ic)) == 0.0)
@@ -1243,11 +1311,8 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
             }
           /* if BITWISEOR then check if one of them is 0xff... */
           /* if yes turn it into 0xff... assignment */
-          {
-            unsigned val;
-
-            switch (getSize (operandType (IC_RIGHT (ic))))
-              {
+          switch (getSize (operandType (IC_RIGHT (ic))))
+            {
               case 1:
                 val = 0xff;
                 break;
@@ -1259,10 +1324,10 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
                 break;
               default:
                 return;
-              }
-            if (((unsigned) double2ul (operandLitValue (IC_RIGHT (ic))) & val) == val)
-              {
-                if (IS_OP_VOLATILE (IC_LEFT (ic)))
+            }
+          if (((unsigned) double2ul (operandLitValue (IC_RIGHT (ic))) & val) == val)
+            {
+              if (IS_OP_VOLATILE (IC_LEFT (ic)))
                 {
                   iCode *newic = newiCode (DUMMY_READ_VOLATILE, NULL, IC_LEFT (ic));
                   IC_RESULT (newic) = IC_LEFT (ic);
@@ -1270,12 +1335,11 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
                   newic->lineno = ic->lineno;
                   addiCodeToeBBlock (ebp, newic, ic->next);
                 }
-                ic->op = '=';
-                IC_LEFT (ic) = NULL;
-                SET_RESULT_RIGHT (ic);
-                return;
-              }
-          }
+              ic->op = '=';
+              IC_LEFT (ic) = NULL;
+              SET_RESULT_RIGHT (ic);
+              return;
+            }
         }
       break;
     case '^':
@@ -1330,6 +1394,7 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
 
   return;
 }
+
 #define OTHERS_PARM(s) (s->_isparm && !s->ismyparm)
 /*-----------------------------------------------------------------*/
 /* updateSpillLocation - keeps track of register spill location    */
@@ -1337,73 +1402,70 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
 void
 updateSpillLocation (iCode * ic, int induction)
 {
-        sym_link *setype;
+  sym_link *setype;
 
-        if (POINTER_SET (ic))
-                return;
+  if (POINTER_SET (ic))
+    return;
 
-        if (ic->nosupdate)
-                return;
+  if (ic->nosupdate)
+    return;
 
 #if 0
-        /* for the form true_symbol := iTempNN */
-        if (ASSIGN_ITEMP_TO_SYM (ic) &&
-            !SPIL_LOC (IC_RIGHT (ic))) {
-
-                setype = getSpec (operandType (IC_RESULT (ic)));
-
-                if (!OP_SYMBOL(IC_RIGHT (ic))->noSpilLoc &&
-                    !IS_VOLATILE (setype) &&
-                    !IN_FARSPACE (SPEC_OCLS (setype)) &&
-                    !OTHERS_PARM (OP_SYMBOL (IC_RESULT (ic))))
-                {
-                    wassert(IS_SYMOP(IC_RESULT (ic)));
-                    wassert(IS_SYMOP(IC_RIGHT (ic)));
-                        SPIL_LOC (IC_RIGHT (ic)) =
-                                IC_RESULT (ic)->operand.symOperand;
-                }
+  /* for the form true_symbol := iTempNN */
+  if (ASSIGN_ITEMP_TO_SYM (ic) &&
+      !SPIL_LOC (IC_RIGHT (ic)))
+    {
+      setype = getSpec (operandType (IC_RESULT (ic)));
 
+      if (!OP_SYMBOL(IC_RIGHT (ic))->noSpilLoc &&
+          !IS_VOLATILE (setype) &&
+          !IN_FARSPACE (SPEC_OCLS (setype)) &&
+          !OTHERS_PARM (OP_SYMBOL (IC_RESULT (ic))))
+        {
+          wassert(IS_SYMOP(IC_RESULT (ic)));
+          wassert(IS_SYMOP(IC_RIGHT (ic)));
+          SPIL_LOC (IC_RIGHT (ic)) = IC_RESULT (ic)->operand.symOperand;
         }
+    }
 #endif
 
 #if 0 /* this needs furthur investigation can save a lot of code */
-        if (ASSIGN_SYM_TO_ITEMP(ic) &&
-            !SPIL_LOC(IC_RESULT(ic))) {
-            if (!OTHERS_PARM (OP_SYMBOL (IC_RIGHT (ic))))
-                SPIL_LOC (IC_RESULT (ic)) =
-                    IC_RIGHT (ic)->operand.symOperand;
-        }
+  if (ASSIGN_SYM_TO_ITEMP(ic) &&
+      !SPIL_LOC(IC_RESULT(ic)))
+    {
+      if (!OTHERS_PARM (OP_SYMBOL (IC_RIGHT (ic))))
+          SPIL_LOC (IC_RESULT (ic)) = IC_RIGHT (ic)->operand.symOperand;
+    }
 #endif
-        if (ASSIGN_ITEMP_TO_ITEMP (ic)) {
+  if (ASSIGN_ITEMP_TO_ITEMP (ic))
+    {
+      if (!SPIL_LOC (IC_RIGHT (ic)) &&
+          !bitVectBitsInCommon (OP_DEFS (IC_RIGHT (ic)), OP_USES (IC_RESULT (ic))) &&
+          OP_SYMBOL (IC_RESULT (ic))->isreqv)
+        {
+          setype = getSpec (operandType (IC_RESULT (ic)));
 
-                if (!SPIL_LOC (IC_RIGHT (ic)) &&
-                    !bitVectBitsInCommon (OP_DEFS (IC_RIGHT (ic)), OP_USES (IC_RESULT (ic))) &&
-                    OP_SYMBOL (IC_RESULT (ic))->isreqv) {
-
-                        setype = getSpec (operandType (IC_RESULT (ic)));
-
-                        if (!OP_SYMBOL(IC_RIGHT (ic))->noSpilLoc &&
-                            !IS_VOLATILE (setype) &&
-                            !IN_FARSPACE (SPEC_OCLS (setype)) &&
-                            !OTHERS_PARM (OP_SYMBOL (IC_RESULT (ic)))) {
-
-                                SPIL_LOC (IC_RIGHT (ic)) =
-                                        SPIL_LOC (IC_RESULT (ic));
-                                OP_SYMBOL (IC_RIGHT (ic))->prereqv =
-                                        OP_SYMBOL (IC_RESULT (ic))->prereqv;
-                        }
-                }
-                /* special case for inductions */
-                if (induction &&
-                    OP_SYMBOL(IC_RIGHT(ic))->isreqv &&
-                    !OP_SYMBOL(IC_RESULT (ic))->noSpilLoc &&
-                    !SPIL_LOC(IC_RESULT(ic))) {
-                        SPIL_LOC (IC_RESULT (ic)) = SPIL_LOC (IC_RIGHT (ic));
-                        OP_SYMBOL (IC_RESULT (ic))->prereqv =
-                                OP_SYMBOL (IC_RIGHT (ic))->prereqv;
-                }
+          if (!OP_SYMBOL(IC_RIGHT (ic))->noSpilLoc &&
+              !IS_VOLATILE (setype) &&
+              !IN_FARSPACE (SPEC_OCLS (setype)) &&
+              !OTHERS_PARM (OP_SYMBOL (IC_RESULT (ic))))
+            {
+              SPIL_LOC (IC_RIGHT (ic)) = SPIL_LOC (IC_RESULT (ic));
+              OP_SYMBOL (IC_RIGHT (ic))->prereqv = OP_SYMBOL (IC_RESULT (ic))->prereqv;
+            }
         }
+      /* special case for inductions */
+      if (induction &&
+          OP_SYMBOL(IC_RIGHT(ic))->isreqv &&
+          !OP_SYMBOL(IC_RESULT (ic))->noSpilLoc &&
+          !SPIL_LOC(IC_RESULT(ic)))
+        {
+          SPIL_LOC (IC_RESULT (ic)) = SPIL_LOC (IC_RIGHT (ic));
+          OP_SYMBOL (IC_RESULT (ic))->prereqv = OP_SYMBOL (IC_RIGHT (ic))->prereqv;
+        }
+    }
 }
+
 /*-----------------------------------------------------------------*/
 /* setUsesDef - sets the uses def bitvector for a given operand    */
 /*-----------------------------------------------------------------*/
@@ -1602,11 +1664,6 @@ constFold (iCode * ic, set * cseSet)
       ic->op != '-')
     return 0;
 
-  /* this check is a heuristic to prevent live ranges
-     from becoming too long */
-  if (IS_PTR (operandType (IC_RESULT (ic))))
-      return 0;
-
   /* check if operation with a literal */
   if (!IS_OP_LITERAL (IC_RIGHT (ic)))
     return 0;
@@ -1650,10 +1707,47 @@ constFold (iCode * ic, set * cseSet)
       OP_SYMBOL(IC_RESULT (ic))->noSpilLoc = 1;
     }
 
-
   return 1;
 }
 
+/* Remove casts to bool from results of logical operations. */
+static int
+boolCast (iCode * ic, set * cseSet)
+{
+  iCode *dic = NULL;
+
+  /* Only casts to booleans are optimized away. */
+  if (ic->op != CAST || !IS_BOOL ( operandType (IC_RESULT (ic))))
+    return 0;
+
+  /* Find the definition for the right hand side. */
+  if (!(applyToSet (cseSet, diCodeForSym, IC_RIGHT (ic), &dic)))
+    return 0;
+
+  /* Check that this is a logic op. */
+  if (dic->op != '!' &&
+      dic->op != '<' &&
+      dic->op != '>' &&
+//      dic->op != LE_OP && why not these 3 ???
+//      dic->op != GE_OP &&
+      dic->op != EQ_OP &&
+//      dic->op != NE_OP &&
+      dic->op != AND_OP &&
+      dic->op != OR_OP &&
+      dic->op != GETHBIT &&
+      dic->op != GETABIT &&
+      !(dic->op == BITWISEAND && (IS_BOOL ( operandType (IC_LEFT (dic))) || IS_BOOL ( operandType (IC_RIGHT (dic)))))
+     )
+    {
+      return 0;
+    }
+
+  /* Replace cast by assignment. */
+  ic->op = '=';
+
+  return 0;
+}
+
 /*-----------------------------------------------------------------*/
 /* deleteGetPointers - called when a pointer is passed as parm     */
 /* will delete from cseSet all get pointers computed from this     */
@@ -1696,8 +1790,8 @@ deleteGetPointers (set ** cseSet, set ** pss, operand * op, eBBlock * ebb)
                   if (!isinSetWith (compItems, (void*)IC_RESULT (cdp->diCode),
                                     (insetwithFunc)isOperandEqual))
                     {
-                  addSet (&compItems, IC_RESULT (cdp->diCode));
-                  changes++;
+                      addSet (&compItems, IC_RESULT (cdp->diCode));
+                      changes++;
                     }
                 }
             }
@@ -1837,8 +1931,7 @@ dumpCseSet(set *cseSet)
 /*             data flow related information is computed by it     */
 /*-----------------------------------------------------------------*/
 int
-cseBBlock (eBBlock * ebb, int computeOnly,
-           ebbIndex * ebbi)
+cseBBlock (eBBlock * ebb, int computeOnly, ebbIndex * ebbi)
 {
   eBBlock ** ebbs = ebbi->bbOrder;
   int count = ebbi->count;
@@ -1897,8 +1990,7 @@ cseBBlock (eBBlock * ebb, int computeOnly,
       if (ic->op == PCALL || ic->op == CALL || ic->op == RECEIVE)
         {
           /* add to defSet of the symbol */
-          OP_DEFS(IC_RESULT (ic))=
-            bitVectSetBit (OP_DEFS (IC_RESULT (ic)), ic->key);
+          OP_DEFS (IC_RESULT (ic)) = bitVectSetBit (OP_DEFS (IC_RESULT (ic)), ic->key);
           /* add to the definition set of this block */
           ebb->defSet = bitVectSetBit (ebb->defSet, ic->key);
           ebb->ldefs = bitVectSetBit (ebb->ldefs, ic->key);
@@ -1931,7 +2023,6 @@ cseBBlock (eBBlock * ebb, int computeOnly,
            ic->op == SEND) &&
           IS_SYMOP (IC_LEFT (ic)))
         {
-
           /* check if they can be replaced */
           if (!computeOnly)
             {
@@ -1949,8 +2040,7 @@ cseBBlock (eBBlock * ebb, int computeOnly,
                            ebb->outDefs, &ebb->usesDefs);
             }
 
-
-          /* if we a sending a pointer as a parameter
+          /* if we are sending a pointer as a parameter
              then kill all cse since the pointed to item
              might be changed in the function being called */
           if ((ic->op == IPUSH || ic->op == SEND) &&
@@ -1986,6 +2076,7 @@ cseBBlock (eBBlock * ebb, int computeOnly,
           /* do some algebraic optimizations if possible */
           algebraicOpts (ic, ebb);
           while (constFold (ic, cseSet));
+          while (boolCast (ic, cseSet));
         }
 
       /* small kludge */
@@ -2027,9 +2118,7 @@ cseBBlock (eBBlock * ebb, int computeOnly,
       /* check if the condition can be replaced */
       if (ic->op == IFX)
         {
-          ifxOptimize (ic, cseSet, computeOnly,
-                       ebb, &change,
-                       ebbi);
+          ifxOptimize (ic, cseSet, computeOnly, ebb, &change, ebbi);
           continue;
         }
 
@@ -2037,9 +2126,8 @@ cseBBlock (eBBlock * ebb, int computeOnly,
       /* see if we can replace it             */
       if (!computeOnly && ic->op == '=')
         {
-
           /* update the spill location for this */
-          updateSpillLocation (ic,0);
+          updateSpillLocation (ic, 0);
 
           if (POINTER_SET (ic) && IS_SYMOP (IC_RESULT (ic)) &&
               !(IS_BITFIELD (OP_SYMBOL (IC_RESULT (ic))->etype)))
@@ -2066,11 +2154,11 @@ cseBBlock (eBBlock * ebb, int computeOnly,
       /* then replace them with those from cseSet */
       /* left operand */
       /* and left is a symbol  */
-      if (IS_SYMOP (IC_LEFT (ic)) &&
-          !IS_BITFIELD (OP_SYM_ETYPE (IC_LEFT (ic))) &&
-          !computeOnly && ic->op != ADDRESS_OF)
+      if (!computeOnly &&
+          ic->op != ADDRESS_OF &&
+          IS_SYMOP (IC_LEFT (ic)) &&
+          !IS_BITFIELD (OP_SYM_ETYPE (IC_LEFT (ic))))
         {
-
           pdop = NULL;
           applyToSetFTrue (cseSet, findCheaperOp, IC_LEFT (ic), &pdop, checkSign);
           if (pdop)
@@ -2079,11 +2167,11 @@ cseBBlock (eBBlock * ebb, int computeOnly,
                 {
                   if (IS_ITEMP (pdop) || IS_OP_LITERAL (pdop))
                     {
-                        /* some non dominating block does POINTER_SET with
-                           this variable .. unsafe to remove any POINTER_GETs */
-                        if (bitVectBitValue(ebb->ndompset,IC_LEFT(ic)->key))
-                            ebb->ptrsSet = bitVectSetBit(ebb->ptrsSet,pdop->key);
-                        ReplaceOpWithCheaperOp(&IC_LEFT(ic), pdop);
+                      /* some non dominating block does POINTER_SET with
+                         this variable .. unsafe to remove any POINTER_GETs */
+                      if (bitVectBitValue (ebb->ndompset, IC_LEFT (ic)->key))
+                          ebb->ptrsSet = bitVectSetBit (ebb->ptrsSet, pdop->key);
+                      ReplaceOpWithCheaperOp (&IC_LEFT (ic), pdop);
                       change = 1;
                     }
                   /* check if there is a pointer set
@@ -2095,29 +2183,42 @@ cseBBlock (eBBlock * ebb, int computeOnly,
                     {
                       ic->op = '=';
                       IC_LEFT (ic) = NULL;
-                      ReplaceOpWithCheaperOp(&IC_RIGHT(ic), pdop);
+                      ReplaceOpWithCheaperOp (&IC_RIGHT (ic), pdop);
                       SET_ISADDR (IC_RESULT (ic), 0);
                     }
-
                 }
               else
                 {
-                  ReplaceOpWithCheaperOp(&IC_LEFT(ic), pdop);
+                  ReplaceOpWithCheaperOp (&IC_LEFT (ic), pdop);
                   change = 1;
                 }
             }
         }
 
       /* right operand */
-      if (IS_SYMOP (IC_RIGHT (ic)) && !computeOnly)
+      if (!computeOnly && IS_SYMOP (IC_RIGHT (ic)))
         {
-
           pdop = NULL;
           applyToSetFTrue (cseSet, findCheaperOp, IC_RIGHT (ic), &pdop, checkSign);
-          if (pdop) {
-            ReplaceOpWithCheaperOp(&IC_RIGHT(ic), pdop);
-            change = 1;
-          }
+          if (pdop)
+            {
+              ReplaceOpWithCheaperOp (&IC_RIGHT (ic), pdop);
+              change = 1;
+            }
+        }
+
+      if (!computeOnly &&
+          POINTER_SET (ic) &&
+          IS_SYMOP (IC_RESULT (ic)) &&
+          !IS_BITFIELD (OP_SYM_ETYPE (IC_RESULT (ic))))
+        {
+          pdop = NULL;
+          applyToSetFTrue (cseSet, findCheaperOp, IC_RESULT (ic), &pdop, checkSign);
+          if (pdop)
+            {
+              ReplaceOpWithCheaperOp (&IC_RESULT (ic), pdop);
+              change = 1;
+            }
         }
 
       /* if left or right changed then do algebraic */
@@ -2152,42 +2253,51 @@ cseBBlock (eBBlock * ebb, int computeOnly,
         {
           applyToSet (cseSet, findPrevIc, ic, &pdic);
           if (pdic && compareType (operandType (IC_RESULT (pdic)),
-                                 operandType (IC_RESULT (ic))) != 1)
-            pdic = NULL;
-          if (pdic && port->cseOk && (*port->cseOk)(ic,pdic) == 0)
+                                   operandType (IC_RESULT (ic))) != 1)
+            {
               pdic = NULL;
+            }
+          if (pdic && port->cseOk && (*port->cseOk)(ic, pdic) == 0)
+            {
+              pdic = NULL;
+            }
         }
 
       /* Alternate code */
-      if (pdic && IS_ITEMP(IC_RESULT(ic))) {
-        if (POINTER_GET(ic) && bitVectBitValue(ebb->ptrsSet,IC_LEFT(ic)->key)) {
-          /* Mmm, found an equivalent pointer get at a lower level.
-             This could be a loop however with the same pointer set
-             later on */
-        } else {
-          /* if previous definition found change this to an assignment */
-          ic->op = '=';
-          IC_LEFT(ic) = NULL;
-          IC_RIGHT(ic) = operandFromOperand(IC_RESULT(pdic));
-          SET_ISADDR(IC_RESULT(ic),0);
-          SET_ISADDR(IC_RIGHT (ic),0);
+      if (pdic && IS_ITEMP (IC_RESULT (ic)))
+        {
+          if (POINTER_GET (ic) && bitVectBitValue (ebb->ptrsSet, IC_LEFT (ic)->key))
+            {
+              /* Mmm, found an equivalent pointer get at a lower level.
+                 This could be a loop however with the same pointer set
+                 later on */
+            }
+          else
+            {
+              /* if previous definition found change this to an assignment */
+              ic->op = '=';
+              IC_LEFT (ic) = NULL;
+              IC_RIGHT (ic) = operandFromOperand (IC_RESULT (pdic));
+              SET_ISADDR (IC_RESULT (ic), 0);
+              SET_ISADDR (IC_RIGHT (ic), 0);
+            }
         }
-      }
 
-      if (!(POINTER_SET (ic)) && IC_RESULT (ic)) {
+      if (!(POINTER_SET (ic)) && IC_RESULT (ic))
+        {
           cseDef *csed;
           deleteItemIf (&cseSet, ifDefSymIsX, IC_RESULT (ic));
           csed = newCseDef (IC_RESULT (ic), ic);
           updateCseDefAncestors (csed, cseSet);
           addSetHead (&cseSet, csed);
-      }
+        }
       defic = ic;
 
       /* if assignment to a parameter which is not
          mine and type is a pointer then delete
          pointerGets to take care of aliasing */
       if (ASSIGNMENT (ic) &&
-                  IS_SYMOP (IC_RESULT (ic)) &&
+          IS_SYMOP (IC_RESULT (ic)) &&
           OTHERS_PARM (OP_SYMBOL (IC_RESULT (ic))) &&
           IS_PTR (operandType (IC_RESULT (ic))))
         {
@@ -2226,25 +2336,29 @@ cseBBlock (eBBlock * ebb, int computeOnly,
       /* add the left & right to the defUse set */
       if (IC_LEFT (ic) && IS_SYMOP (IC_LEFT (ic)))
         {
-          OP_USES(IC_LEFT (ic))=
+          OP_USES (IC_LEFT (ic)) =
             bitVectSetBit (OP_USES (IC_LEFT (ic)), ic->key);
           setUsesDefs (IC_LEFT (ic), ebb->defSet, ebb->outDefs, &ebb->usesDefs);
         }
 
       if (IC_RIGHT (ic) && IS_SYMOP (IC_RIGHT (ic)))
         {
-          OP_USES(IC_RIGHT (ic))=
+          OP_USES (IC_RIGHT (ic)) =
             bitVectSetBit (OP_USES (IC_RIGHT (ic)), ic->key);
           setUsesDefs (IC_RIGHT (ic), ebb->defSet, ebb->outDefs, &ebb->usesDefs);
         }
 
       /* for the result it is special case, put the result */
-      /* in the defuseSet if it a pointer or array access  */
-      if (POINTER_SET (defic) && IS_SYMOP (IC_RESULT (ic)))
+      /* in the defuseSet if it is a pointer or array access */
+      if (POINTER_SET (defic) &&
+		  (IS_SYMOP (IC_RESULT (ic)) || IS_OP_LITERAL (IC_RESULT (ic))))
         {
-          OP_USES(IC_RESULT (ic))=
-            bitVectSetBit (OP_USES (IC_RESULT (ic)), ic->key);
-          setUsesDefs (IC_RESULT (ic), ebb->defSet, ebb->outDefs, &ebb->usesDefs);
+          if (IS_SYMOP (IC_RESULT (ic)))
+            {
+              OP_USES (IC_RESULT (ic)) =
+                bitVectSetBit (OP_USES (IC_RESULT (ic)), ic->key);
+              setUsesDefs (IC_RESULT (ic), ebb->defSet, ebb->outDefs, &ebb->usesDefs);
+            }
           deleteItemIf (&cseSet, ifPointerGet, IC_RESULT (ic));
           ebb->ptrsSet = bitVectSetBit (ebb->ptrsSet, IC_RESULT (ic)->key);
           /* delete from inexpressions of all successors which
@@ -2283,9 +2397,10 @@ cseBBlock (eBBlock * ebb, int computeOnly,
 
   for (expr=setFirstItem (ebb->inExprs); expr; expr=setNextItem (ebb->inExprs))
     if (!isinSetWith (cseSet, expr, isCseDefEqual) &&
-        !isinSetWith (ebb->killedExprs, expr, isCseDefEqual)) {
-    addSetHead (&ebb->killedExprs, expr);
-  }
+        !isinSetWith (ebb->killedExprs, expr, isCseDefEqual))
+      {
+        addSetHead (&ebb->killedExprs, expr);
+      }
   setToNull ((void *) &ebb->outExprs);
   ebb->outExprs = cseSet;
   ebb->outDefs = bitVectUnion (ebb->outDefs, ebb->defSet);
@@ -2293,9 +2408,9 @@ cseBBlock (eBBlock * ebb, int computeOnly,
   return change;
 }
 
-/*-----------------------------------------------------------------*/
+/*------------------------------------------------------------------*/
 /* cseAllBlocks - will sequentially go thru & do cse for all blocks */
-/*-----------------------------------------------------------------*/
+/*------------------------------------------------------------------*/
 int
 cseAllBlocks (ebbIndex * ebbi, int computeOnly)
 {
@@ -2311,3 +2426,4 @@ cseAllBlocks (ebbIndex * ebbi, int computeOnly)
 
   return change;
 }
+
