@@ -74,82 +74,82 @@ cnvToFcall (iCode * ic, eBBlock * ebp)
   if (IS_SYMOP (right))
       bitVectUnSetBit (OP_USES (right), ic->key);
 
-  if (IS_FLOAT (operandType (right))) {
-    switch (ic->op)
-      {
-      case '+':
-        func = __fsadd;
-        break;
-      case '-':
-        func = __fssub;
-        break;
-      case '/':
-        func = __fsdiv;
-        break;
-      case '*':
-        func = __fsmul;
-        break;
-      case EQ_OP:
-        func = __fseq;
-        break;
-      case NE_OP:
-        func = __fsneq;
-        break;
-      case '<':
-        func = __fslt;
-        break;
-      case '>':
-        func = __fsgt;
-        break;
-      case LE_OP:
-        func = __fslteq;
-        break;
-      case GE_OP:
-        func = __fsgteq;
-        break;
-      }
-  } else
-  if (IS_FIXED16X16 (operandType (right))) {
-    switch (ic->op)
-      {
-      case '+':
-        func = __fps16x16_add;
-        break;
-      case '-':
-        func = __fps16x16_sub;
-        break;
-      case '/':
-        func = __fps16x16_div;
-        break;
-      case '*':
-        func = __fps16x16_mul;
-        break;
-      case EQ_OP:
-        func = __fps16x16_eq;
-        break;
-      case NE_OP:
-        func = __fps16x16_neq;
-        break;
-      case '<':
-        func = __fps16x16_lt;
-        break;
-      case '>':
-        func = __fps16x16_gt;
-        break;
-      case LE_OP:
-        func = __fps16x16_lteq;
-        break;
-      case GE_OP:
-        func = __fps16x16_gteq;
-        break;
-      }
-  }
-
+  if (IS_FLOAT (operandType (right)))
+    {
+      switch (ic->op)
+        {
+        case '+':
+          func = fsadd;
+          break;
+        case '-':
+          func = fssub;
+          break;
+        case '/':
+          func = fsdiv;
+          break;
+        case '*':
+          func = fsmul;
+          break;
+        case EQ_OP:
+          func = fseq;
+          break;
+        case NE_OP:
+          func = fsneq;
+          break;
+        case '<':
+          func = fslt;
+          break;
+        case '>':
+          func = fsgt;
+          break;
+        case LE_OP:
+          func = fslteq;
+          break;
+        case GE_OP:
+          func = fsgteq;
+          break;
+        }
+    }
+  else if (IS_FIXED16X16 (operandType (right)))
+    {
+      switch (ic->op)
+        {
+        case '+':
+          func = fps16x16_add;
+          break;
+        case '-':
+          func = fps16x16_sub;
+          break;
+        case '/':
+          func = fps16x16_div;
+          break;
+        case '*':
+          func = fps16x16_mul;
+          break;
+        case EQ_OP:
+          func = fps16x16_eq;
+          break;
+        case NE_OP:
+          func = fps16x16_neq;
+          break;
+        case '<':
+          func = fps16x16_lt;
+          break;
+        case '>':
+          func = fps16x16_gt;
+          break;
+        case LE_OP:
+          func = fps16x16_lteq;
+          break;
+        case GE_OP:
+          func = fps16x16_gteq;
+          break;
+        }
+    }
 
   /* if float support routines NOT compiled as reentrant */
   if (!options.float_rent)
     {
-
       /* first one */
       if (IS_REGPARM (FUNC_ARGS(func->type)->etype))
         {
@@ -184,11 +184,9 @@ cnvToFcall (iCode * ic, eBBlock * ebp)
       newic->lineno = lineno;
       if (IS_SYMOP (right))
           OP_USES (right) = bitVectSetBit (OP_USES (right), newic->key);
-
     }
   else
     {
-
       /* push right */
       if (IS_REGPARM (FUNC_ARGS(func->type)->next->etype))
         {
@@ -224,8 +222,7 @@ cnvToFcall (iCode * ic, eBBlock * ebp)
       newic->filename = filename;
       newic->lineno = lineno;
       if (IS_SYMOP (left))
-          OP_USES (left) = bitVectSetBit (OP_USES (left), newic->key);
-
+        OP_USES (left) = bitVectSetBit (OP_USES (left), newic->key);
     }
   /* insert the call */
   newic = newiCode (CALL, operandFromSymbol (func), NULL);
@@ -239,18 +236,20 @@ cnvToFcall (iCode * ic, eBBlock * ebp)
   if (currFunc)
     FUNC_HASFCALL (currFunc->type) = 1;
 
-  if(TARGET_IS_PIC16 || TARGET_IS_PIC) {
-        /* normally these functions aren't marked external, so we can use their
-         * _extern field to marked as already added to symbol table */
+  if (TARGET_PIC_LIKE)
+    {
+      /* normally these functions aren't marked external, so we can use their
+       * _extern field to mark as already added to symbol table */
 
-        if(!SPEC_EXTR(func->etype)) {
-            memmap *seg = SPEC_OCLS(OP_SYMBOL(IC_LEFT(newic))->etype);
+      if (!SPEC_EXTR(func->etype))
+        {
+          memmap *seg = SPEC_OCLS(OP_SYMBOL(IC_LEFT(newic))->etype);
 
-            SPEC_EXTR(func->etype) = 1;
-            seg = SPEC_OCLS( func->etype );
-            addSet(&seg->syms, func);
+          SPEC_EXTR(func->etype) = 1;
+          seg = SPEC_OCLS( func->etype );
+          addSet(&seg->syms, func);
         }
-  }
+    }
 
   addiCodeToeBBlock (ebp, newic, ip);
 }
@@ -276,18 +275,19 @@ cnvToFloatCast (iCode * ic, eBBlock * ebp)
     {
       for (su = 0; su < 2; su++)
         {
-          if (compareType (type, __multypes[bwd][su]) == 1)
+          if (compareType (type, multypes[bwd][su]) == 1)
             {
-              func = __conv[0][bwd][su];
+              func = conv[0][bwd][su];
               goto found;
             }
         }
     }
 
-  if(compareType (type, fixed16x16Type) == 1) {
-    func = __fp16x16conv[0][3][0];
-    goto found;
-  }
+  if (compareType (type, fixed16x16Type) == 1)
+    {
+      func = fp16x16conv[0][3][0];
+      goto found;
+    }
 
   assert (0);
 found:
@@ -309,21 +309,21 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = linenno;
-
     }
   else
     {
       /* push the left */
-      if (IS_REGPARM (FUNC_ARGS(func->type)->etype)) {
-        newic = newiCode (SEND, IC_RIGHT (ic), NULL);
-        newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
-      }
+      if (IS_REGPARM (FUNC_ARGS(func->type)->etype))
+        {
+          newic = newiCode (SEND, IC_RIGHT (ic), NULL);
+          newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
+        }
       else
-      {
-        newic = newiCode (IPUSH, IC_RIGHT (ic), NULL);
-        newic->parmPush = 1;
-        bytesPushed += getSize(operandType(IC_RIGHT(ic)));
-      }
+        {
+          newic = newiCode (IPUSH, IC_RIGHT (ic), NULL);
+          newic->parmPush = 1;
+          bytesPushed += getSize(operandType(IC_RIGHT(ic)));
+        }
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = linenno;
@@ -337,18 +337,20 @@ found:
   if (currFunc)
     FUNC_HASFCALL (currFunc->type) = 1;
 
-  if(TARGET_IS_PIC16 || TARGET_IS_PIC) {
-        /* normally these functions aren't marked external, so we can use their
-         * _extern field to marked as already added to symbol table */
+  if (TARGET_PIC_LIKE)
+    {
+      /* normally these functions aren't marked external, so we can use their
+       * _extern field to marked as already added to symbol table */
 
-        if(!SPEC_EXTR(func->etype)) {
-            memmap *seg = SPEC_OCLS(OP_SYMBOL(IC_LEFT(newic))->etype);
+      if (!SPEC_EXTR(func->etype))
+        {
+          memmap *seg = SPEC_OCLS(OP_SYMBOL(IC_LEFT(newic))->etype);
 
-            SPEC_EXTR(func->etype) = 1;
-            seg = SPEC_OCLS( func->etype );
-            addSet(&seg->syms, func);
+          SPEC_EXTR(func->etype) = 1;
+          seg = SPEC_OCLS( func->etype );
+          addSet(&seg->syms, func);
         }
-  }
+    }
 
   addiCodeToeBBlock (ebp, newic, ip);
   newic->filename = filename;
@@ -376,9 +378,9 @@ cnvToFixed16x16Cast (iCode * ic, eBBlock * ebp)
     {
       for (su = 0; su < 2; su++)
         {
-          if (compareType (type, __multypes[bwd][su]) == 1)
+          if (compareType (type, multypes[bwd][su]) == 1)
             {
-              func = __fp16x16conv[0][bwd][su];
+              func = fp16x16conv[0][bwd][su];
               goto found;
             }
         }
@@ -390,11 +392,11 @@ found:
   if (!options.float_rent)
     {
       /* first one */
-        if (IS_REGPARM (FUNC_ARGS(func->type)->etype))
-            {
-                newic = newiCode (SEND, IC_RIGHT (ic), NULL);
-                newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
-            }
+      if (IS_REGPARM (FUNC_ARGS(func->type)->etype))
+        {
+          newic = newiCode (SEND, IC_RIGHT (ic), NULL);
+          newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
+        }
       else
         {
           newic = newiCode ('=', NULL, IC_RIGHT (ic));
@@ -403,14 +405,14 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = linenno;
-
     }
   else
     {
       /* push the left */
-        if (IS_REGPARM (FUNC_ARGS(func->type)->etype)) {
-            newic = newiCode (SEND, IC_RIGHT (ic), NULL);
-            newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
+      if (IS_REGPARM (FUNC_ARGS(func->type)->etype))
+        {
+          newic = newiCode (SEND, IC_RIGHT (ic), NULL);
+          newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
         }
       else
         {
@@ -431,18 +433,20 @@ found:
   if (currFunc)
     FUNC_HASFCALL (currFunc->type) = 1;
 
-  if(TARGET_IS_PIC16 || TARGET_IS_PIC) {
-        /* normally these functions aren't marked external, so we can use their
-         * _extern field to marked as already added to symbol table */
+  if (TARGET_PIC_LIKE)
+    {
+      /* normally these functions aren't marked external, so we can use their
+       * _extern field to marked as already added to symbol table */
 
-        if(!SPEC_EXTR(func->etype)) {
-            memmap *seg = SPEC_OCLS(OP_SYMBOL(IC_LEFT(newic))->etype);
+      if (!SPEC_EXTR(func->etype))
+        {
+          memmap *seg = SPEC_OCLS(OP_SYMBOL(IC_LEFT(newic))->etype);
 
-                SPEC_EXTR(func->etype) = 1;
-                seg = SPEC_OCLS( func->etype );
-                addSet(&seg->syms, func);
+          SPEC_EXTR(func->etype) = 1;
+          seg = SPEC_OCLS( func->etype );
+          addSet(&seg->syms, func);
         }
-  }
+    }
 
   addiCodeToeBBlock (ebp, newic, ip);
   newic->filename = filename;
@@ -472,9 +476,9 @@ cnvFromFloatCast (iCode * ic, eBBlock * ebp)
     {
       for (su = 0; su < 2; su++)
         {
-          if (compareType (type, __multypes[bwd][su]) == 1)
+          if (compareType (type, multypes[bwd][su]) == 1)
             {
-              func = __conv[1][bwd][su];
+              func = conv[1][bwd][su];
               goto found;
             }
         }
@@ -486,9 +490,10 @@ found:
   if (!options.float_rent)
     {
       /* first one */
-        if (IS_REGPARM (FUNC_ARGS(func->type)->etype)) {
-            newic = newiCode (SEND, IC_RIGHT (ic), NULL);
-            newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
+      if (IS_REGPARM (FUNC_ARGS(func->type)->etype))
+        {
+          newic = newiCode (SEND, IC_RIGHT (ic), NULL);
+          newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
         }
       else
         {
@@ -498,15 +503,14 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = lineno;
-
     }
   else
     {
-
       /* push the left */
-        if (IS_REGPARM (FUNC_ARGS(func->type)->etype)) {
-            newic = newiCode (SEND, IC_RIGHT (ic), NULL);
-            newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
+      if (IS_REGPARM (FUNC_ARGS(func->type)->etype))
+        {
+          newic = newiCode (SEND, IC_RIGHT (ic), NULL);
+          newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
         }
       else
         {
@@ -517,7 +521,6 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = lineno;
-
     }
 
   /* make the call */
@@ -528,18 +531,20 @@ found:
   if (currFunc)
     FUNC_HASFCALL (currFunc->type) = 1;
 
-  if(TARGET_IS_PIC16 || TARGET_IS_PIC) {
-        /* normally these functions aren't marked external, so we can use their
-         * _extern field to marked as already added to symbol table */
+  if (TARGET_PIC_LIKE)
+    {
+      /* normally these functions aren't marked external, so we can use their
+       * _extern field to marked as already added to symbol table */
 
-        if(!SPEC_EXTR(func->etype)) {
-            memmap *seg = SPEC_OCLS(OP_SYMBOL(IC_LEFT(newic))->etype);
+      if (!SPEC_EXTR(func->etype))
+        {
+          memmap *seg = SPEC_OCLS(OP_SYMBOL(IC_LEFT(newic))->etype);
 
-                SPEC_EXTR(func->etype) = 1;
-                seg = SPEC_OCLS( func->etype );
-                addSet(&seg->syms, func);
+          SPEC_EXTR(func->etype) = 1;
+          seg = SPEC_OCLS( func->etype );
+          addSet(&seg->syms, func);
         }
-  }
+    }
 
   addiCodeToeBBlock (ebp, newic, ip);
   newic->filename = filename;
@@ -569,9 +574,9 @@ cnvFromFixed16x16Cast (iCode * ic, eBBlock * ebp)
     {
       for (su = 0; su < 2; su++)
         {
-          if (compareType (type, __multypes[bwd][su]) == 1)
+          if (compareType (type, multypes[bwd][su]) == 1)
             {
-              func = __fp16x16conv[1][bwd][su];
+              func = fp16x16conv[1][bwd][su];
               goto found;
             }
         }
@@ -579,7 +584,7 @@ cnvFromFixed16x16Cast (iCode * ic, eBBlock * ebp)
 
   if (compareType (type, floatType) == 1)
     {
-      func = __fp16x16conv[1][3][0];
+      func = fp16x16conv[1][3][0];
       goto found;
     }
 
@@ -590,9 +595,10 @@ found:
   if (!options.float_rent)
     {
       /* first one */
-        if (IS_REGPARM (FUNC_ARGS(func->type)->etype)) {
-            newic = newiCode (SEND, IC_RIGHT (ic), NULL);
-            newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
+      if (IS_REGPARM (FUNC_ARGS(func->type)->etype))
+        {
+          newic = newiCode (SEND, IC_RIGHT (ic), NULL);
+          newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
         }
       else
         {
@@ -605,11 +611,11 @@ found:
     }
   else
     {
-
       /* push the left */
-        if (IS_REGPARM (FUNC_ARGS(func->type)->etype)) {
-            newic = newiCode (SEND, IC_RIGHT (ic), NULL);
-            newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
+      if (IS_REGPARM (FUNC_ARGS(func->type)->etype))
+        {
+          newic = newiCode (SEND, IC_RIGHT (ic), NULL);
+          newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
         }
       else
         {
@@ -630,18 +636,20 @@ found:
   if (currFunc)
     FUNC_HASFCALL (currFunc->type) = 1;
 
-  if(TARGET_IS_PIC16 || TARGET_IS_PIC) {
-        /* normally these functions aren't marked external, so we can use their
-         * _extern field to marked as already added to symbol table */
+  if (TARGET_PIC_LIKE)
+    {
+      /* normally these functions aren't marked external, so we can use their
+       * _extern field to marked as already added to symbol table */
 
-        if(!SPEC_EXTR(func->etype)) {
-            memmap *seg = SPEC_OCLS(OP_SYMBOL(IC_LEFT(newic))->etype);
+      if (!SPEC_EXTR(func->etype))
+        {
+          memmap *seg = SPEC_OCLS(OP_SYMBOL(IC_LEFT(newic))->etype);
 
-                SPEC_EXTR(func->etype) = 1;
-                seg = SPEC_OCLS( func->etype );
-                addSet(&seg->syms, func);
+          SPEC_EXTR(func->etype) = 1;
+          seg = SPEC_OCLS( func->etype );
+          addSet(&seg->syms, func);
         }
-  }
+    }
 
   addiCodeToeBBlock (ebp, newic, ip);
   newic->filename = filename;
@@ -654,8 +662,9 @@ extern operand *geniCodeRValue (operand *, bool);
 /* convilong - converts int or long mults or divs to fcalls        */
 /*-----------------------------------------------------------------*/
 static void
-convilong (iCode * ic, eBBlock * ebp, sym_link * type, int op)
+convilong (iCode * ic, eBBlock * ebp)
 {
+  int op = ic->op;
   symbol *func = NULL;
   iCode *ip = ic->next;
   iCode *newic;
@@ -664,45 +673,81 @@ convilong (iCode * ic, eBBlock * ebp, sym_link * type, int op)
   int bwd;
   int su;
   int bytesPushed=0;
+  sym_link *leftType = operandType (IC_LEFT (ic));
+  sym_link *rightType = operandType (IC_RIGHT (ic));
 
   remiCodeFromeBBlock (ebp, ic);
 
+  if (getSize (leftType) == 1 && getSize (rightType) == 1)
+    {
+      int muldivmod;
+
+      if (op == '*')
+        muldivmod = 0;
+      else if (op == '/')
+        muldivmod = 1;
+      else if (op == '%')
+        muldivmod = 2;
+      else
+        muldivmod = -1;
+
+      for (su = 0; su < 4 && muldivmod >= 0; su++)
+        {
+          if ((compareType (leftType, multypes[0][su%2]) == 1) &&
+              (compareType (rightType, multypes[0][su/2]) == 1))
+            {
+              func = muldiv[muldivmod][0][su];
+              goto found;
+            }
+        }
+    }
+
   /* depending on the type */
   for (bwd = 0; bwd < 3; bwd++)
     {
       for (su = 0; su < 2; su++)
         {
-          if (compareType (type, __multypes[bwd][su]) == 1)
+          if (compareType (leftType, multypes[bwd][su]) == 1)
             {
+              if ((op=='*' || op=='/' || op=='%'))
+                {
+                  int ret = compareType (rightType, multypes[bwd][su]);
+                  if (ret != 1)
+                    {
+                      assert(0);
+                    }
+                }
               if (op == '*')
-                func = __muldiv[0][bwd][su];
+                func = muldiv[0][bwd][su];
               else if (op == '/')
-                func = __muldiv[1][bwd][su];
+                func = muldiv[1][bwd][su];
               else if (op == '%')
-                func = __muldiv[2][bwd][su];
+                func = muldiv[2][bwd][su];
               else if (op == RRC)
-                func = __rlrr[1][bwd][su];
+                func = rlrr[1][bwd][su];
               else if (op == RLC)
-                func = __rlrr[0][bwd][su];
+                func = rlrr[0][bwd][su];
               else if (op == RIGHT_OP)
-                func = __rlrr[1][bwd][su];
+                func = rlrr[1][bwd][su];
               else if (op == LEFT_OP)
-                func = __rlrr[0][bwd][su];
+                func = rlrr[0][bwd][su];
               else
                 assert (0);
               goto found;
             }
         }
     }
-  assert (0);
+  werror (E_INVALID_OP);
+  return;
 found:
   /* if int & long support routines NOT compiled as reentrant */
   if (!options.intlong_rent)
     {
       /* first one */
-        if (IS_REGPARM (FUNC_ARGS(func->type)->etype)) {
-            newic = newiCode (SEND, IC_LEFT (ic), NULL);
-            newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
+      if (IS_REGPARM (FUNC_ARGS(func->type)->etype))
+        {
+          newic = newiCode (SEND, IC_LEFT (ic), NULL);
+          newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->etype);
         }
       else
         {
@@ -714,10 +759,11 @@ found:
       newic->lineno = lineno;
 
       /* second one */
-      if (IS_REGPARM (FUNC_ARGS(func->type)->next->etype)) {
+      if (IS_REGPARM (FUNC_ARGS(func->type)->next->etype))
+        {
           newic = newiCode (SEND, IC_RIGHT (ic), NULL);
           newic->argreg = SPEC_ARGREG(FUNC_ARGS(func->type)->next->etype);
-      }
+        }
       else
         {
           newic = newiCode ('=', NULL, IC_RIGHT (ic));
@@ -726,7 +772,6 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = lineno;
-
     }
   else
     {
@@ -764,7 +809,6 @@ found:
       addiCodeToeBBlock (ebp, newic, ip);
       newic->filename = filename;
       newic->lineno = lineno;
-
     }
 
   /* for the result */
@@ -777,22 +821,130 @@ found:
   if (currFunc)
     FUNC_HASFCALL (currFunc->type) = 1;
 
-  if(TARGET_IS_PIC || TARGET_IS_PIC16) {
-        /* normally these functions aren't marked external, so we can use their
-         * _extern field to marked as already added to symbol table */
+  if (TARGET_PIC_LIKE)
+    {
+      /* normally these functions aren't marked external, so we can use their
+       * _extern field to marked as already added to symbol table */
 
-        if(!SPEC_EXTR(func->etype)) {
-            memmap *seg = SPEC_OCLS(OP_SYMBOL(IC_LEFT(newic))->etype);
+      if (!SPEC_EXTR(func->etype))
+        {
+          memmap *seg = SPEC_OCLS(OP_SYMBOL(IC_LEFT(newic))->etype);
 
-                SPEC_EXTR(func->etype) = 1;
-                seg = SPEC_OCLS( func->etype );
-                addSet(&seg->syms, func);
+          SPEC_EXTR(func->etype) = 1;
+          seg = SPEC_OCLS( func->etype );
+          addSet(&seg->syms, func);
         }
-  }
+    }
 
   addiCodeToeBBlock (ebp, newic, ip);
 }
 
+/*-----------------------------------------------------------------*/
+/* convertbuiltin - maybe convert some builtins back               */
+/*-----------------------------------------------------------------*/
+static void
+convbuiltin (iCode *const ic, eBBlock *ebp)
+{
+  sym_link *ftype;
+  symbol *bif;
+  int stack;
+
+  iCode *icc = ic, *icp = ic->prev, *ico = NULL;
+  iCode *lastparam = ic;
+  while (icc->op != CALL)
+    {
+      if (icc->op != SEND || !icc->builtinSEND)
+        return;
+      lastparam = icc;
+      icc = icc->next;
+    }
+
+  if (!IS_SYMOP (IC_LEFT(icc)))
+    return;
+
+  ftype = operandType (IC_LEFT(icc));
+  if (!IFFUNC_ISBUILTIN (ftype))
+    return;
+
+  bif = OP_SYMBOL (IC_LEFT (icc));
+
+  /* Now we can be sure to have found a builtin function. */
+
+  if ((TARGET_IS_Z80 || TARGET_IS_Z180 || TARGET_IS_R2K) && (!strcmp (bif->name, "__builtin_memcpy") || !strcmp (bif->name, "__builtin_memset")))
+    {
+      /* Replace iff return value is used or last parameter is not an integer constant. */
+      if (bitVectIsZero (OP_USES (IC_RESULT (icc))) && IS_OP_LITERAL (IC_LEFT (lastparam)))
+        return;
+      
+      strcpy(OP_SYMBOL (IC_LEFT (icc))->rname, !strcmp (bif->name, "__builtin_memcpy") ? "_memcpy" : "_memset");
+    }
+  else
+    return;
+
+  /* Convert parameter passings from SEND to PUSH. */
+  stack = 0;
+  for (icc = ic; icc->op != CALL; icc = icc->next)
+    {
+      icc->builtinSEND = 0;
+      icc->op = IPUSH;
+      icc->parmPush = 1;
+      stack += getSize (operandType (IC_LEFT (icc)));
+    }
+  icc->parmBytes = stack;
+
+  /* Reverse parameters. */
+  for (icc = ic; icc->op != CALL; icc = icc->next)
+    {
+      if(icc->next->op != CALL)
+        icc->prev = icc->next;
+      else
+        icc->prev = icp;
+    }
+  if(icc != ic)
+    {
+      if(icp)
+        icp->next = icc->prev;
+      icc->prev = ic;
+    }
+  for(; icc != icp; ico = icc, icc = icc->prev)
+    {
+      if(icc->op != CALL)
+        icc->next = ico;
+    }
+}
+
+static void
+convsmallc (iCode *ic, eBBlock *ebp)
+{
+  iCode *icc, *icp, *ico;
+
+  assert (ic->op == CALL || ic->op == PCALL);
+
+  for(icc = ic->prev; icc && icc->op == IPUSH; icc = icc->prev);
+  icp = icc;
+  ic = icp->next;
+
+  /* Reverse parameters. */
+  for (icc = ic; icc->op != CALL && icc->op != PCALL; icc = icc->next)
+    {
+      if(icc->next->op != CALL && icc->next->op != PCALL)
+        icc->prev = icc->next;
+      else
+        icc->prev = icp;
+    }
+  if(icc != ic)
+    {
+      if(icp)
+        icp->next = icc->prev;
+      icc->prev = ic;
+    }
+  for(; icc != icp; ico = icc, icc = icc->prev)
+    {
+      if(icc->op != CALL && icc->op != PCALL)
+        icc->next = ico;
+    }
+}
+
 /*-----------------------------------------------------------------*/
 /* convertToFcall - converts some operations to fcalls             */
 /*-----------------------------------------------------------------*/
@@ -809,13 +961,10 @@ convertToFcall (eBBlock ** ebbs, int count)
       /* for all instructions in the block do */
       for (ic = ebbs[i]->sch; ic; ic = ic->next)
         {
-
           /* floating point operations are
              converted to function calls */
-          if ((IS_CONDITIONAL (ic) ||
-               IS_ARITHMETIC_OP (ic)) &&
-              (IS_FLOAT (operandType (IC_RIGHT (ic))) ||
-               IS_FIXED( operandType (IC_RIGHT (ic)))))
+          if ((IS_CONDITIONAL (ic) || IS_ARITHMETIC_OP (ic)) &&
+              (IS_FLOAT (operandType (IC_RIGHT (ic))) || IS_FIXED( operandType (IC_RIGHT (ic)))))
             {
               cnvToFcall (ic, ebbs[i]);
             }
@@ -883,7 +1032,7 @@ convertToFcall (eBBlock ** ebbs, int count)
                     }
                   else
                     {
-                      convilong (ic, ebbs[i], leftType, ic->op);
+                      convilong (ic, ebbs[i]);
                     }
                 }
             }
@@ -894,13 +1043,126 @@ convertToFcall (eBBlock ** ebbs, int count)
 
               if (IS_INTEGRAL (type) && getSize (type) > port->support.shift && port->support.shift >= 0)
                 {
-                  convilong (ic, ebbs[i], type, ic->op);
+                  convilong (ic, ebbs[i]);
                 }
             }
+          if (ic->op == SEND && ic->builtinSEND)
+            {
+              convbuiltin (ic, ebbs[i]);
+            }
+          if ((ic->op == CALL  && IFFUNC_ISSMALLC (operandType (IC_LEFT (ic)))) ||
+              (ic->op == PCALL && IFFUNC_ISSMALLC (operandType (IC_LEFT (ic))->next)))
+            {
+              convsmallc (ic, ebbs[i]);
+            }
         }
     }
 }
 
+/*-----------------------------------------------------------------*/
+/* isPowerOf2 - test if val is power of 2                          */
+/*-----------------------------------------------------------------*/
+static bool
+isPowerOf2 (unsigned long val)
+{
+  while (val && !(val & 1))
+    {
+      val >>= 1;
+    }
+  return val == 1;
+}
+
+/*-----------------------------------------------------------------*/
+/* miscOpt - miscellaneous optimizations                           */
+/*-----------------------------------------------------------------*/
+static void
+miscOpt (eBBlock ** ebbs, int count)
+{
+  int i;
+
+  /* for all blocks do */
+  for (i = 0; i < count; ++i)
+    {
+      iCode *ic;
+
+      /* for all instructions in the block do */
+      for (ic = ebbs[i]->sch; ic; ic = ic->next)
+        {
+          /* patch ID: 2702889 - Summary of all uncommitted changes I applied on "my" SDCC */
+          /* MB: This seems rather incomplete.
+             MB: What if using <= or >= ?
+             Borut: Currently <= and >= are transformed to > and < on all targets.
+               Transformation depends on lt_nge, gt_nle, bool le_ngt,
+               ge_nlt, ne_neq and eq_nne members of PORT structure.
+             MB: Why do we need IFX in the first case and not in the second ?
+             Borutr: Because the result of comparision is logically negated,
+               so in case of IFX the jump logic is inverted for '<' and '<='.
+               TODO: The logical negation of the result should be implemeted
+               for '<' and '<=' in case when the following instruction is not IFX.
+             Philipp: Added the test for ifx in the second case, too:
+             We want 0 or 1 as a result, the bitwise and won't do, unless we add a cast to bool.
+          */
+          switch (ic->op)
+            {
+            case '<':
+            case LE_OP:
+            case '>':
+            case GE_OP:
+              /* Only if the the right operand is literal and left operant is unsigned */
+              if (isOperandLiteral (IC_RIGHT (ic)) && IS_UNSIGNED (operandType (IC_LEFT (ic))))
+                {
+                  unsigned litVal = ulFromVal (OP_VALUE (IC_RIGHT (ic)));
+
+                  /* Only if the literal value is greater than 255 and a power of 2 */
+                  if (litVal >= 255 &&
+                    (isPowerOf2 (litVal) && (ic->op == '<' || ic->op == GE_OP) ||
+                    isPowerOf2 (litVal + 1) && (ic->op == '>' || ic->op == LE_OP)))
+                    {
+                      iCode *ic_nxt = ic->next;
+
+                      switch (ic->op)
+                        {
+                        case LE_OP:
+                          ++litVal;
+                          /* fall through */
+                        case '<':
+                          /* Only if the next instruction is IFX */
+                          if (ic_nxt && (ic_nxt->op == IFX) && (ic->eBBlockNum == ic_nxt->eBBlockNum))
+                            {
+                              int AndMaskVal = 0 - litVal;
+                              symbol *TrueLabel;
+
+                              /* set op to bitwise and */
+                              ic->op = BITWISEAND;
+                              IC_RIGHT (ic) = operandFromLit (AndMaskVal);
+
+                              /* invert jump logic */
+                              TrueLabel = IC_TRUE (ic_nxt);
+                              IC_TRUE (ic_nxt) = IC_FALSE (ic_nxt);
+                              IC_FALSE (ic_nxt) = TrueLabel;
+                            }
+                          break;
+
+                        case '>':
+                          ++litVal;
+                          /* fall through */
+                        case GE_OP:
+                            if (ic_nxt && (ic_nxt->op == IFX) && (ic->eBBlockNum == ic_nxt->eBBlockNum))
+                            {
+                              int AndMaskVal = 0 - litVal;
+
+                              ic->op = BITWISEAND;
+                              IC_RIGHT (ic) = operandFromLit (AndMaskVal);
+                            }
+                          break;
+                        } /* switch */
+                    } /* if */
+                } /* if */
+            } /* switch */
+        } /* for */
+    } /* for */
+}
+
 /*-----------------------------------------------------------------*/
 /* isLocalWithoutDef - return 1 if sym might be used without a     */
 /*                     defining iCode                              */
@@ -944,12 +1206,10 @@ replaceRegEqv (ebbIndex * ebbi)
 
   for (i = 0; i < count; i++)
     {
-
       iCode *ic;
 
       for (ic = ebbs[i]->sch; ic; ic = ic->next)
         {
-
           if (SKIP_IC2 (ic))
             continue;
 
@@ -966,16 +1226,13 @@ replaceRegEqv (ebbIndex * ebbi)
                   OP_SYMBOL (IC_COND (ic))->allocreq = 1;
                 }
 
-              if (IS_TRUE_SYMOP (IC_COND (ic)) &&
-                  OP_REQV (IC_COND (ic)))
+              if (IS_TRUE_SYMOP (IC_COND (ic)) && OP_REQV (IC_COND (ic)))
                 IC_COND (ic) = opFromOpWithDU (OP_REQV (IC_COND (ic)),
-                                             OP_SYMBOL (IC_COND (ic))->defs,
-                                            OP_SYMBOL (IC_COND (ic))->uses);
-
+                                               OP_DEFS (IC_COND (ic)),
+                                               OP_USES (IC_COND (ic)));
               continue;
             }
 
-
           if (ic->op == JUMPTABLE)
             {
               if (IC_JTCOND (ic) &&
@@ -989,11 +1246,10 @@ replaceRegEqv (ebbIndex * ebbi)
                   OP_SYMBOL (IC_JTCOND (ic))->allocreq = 1;
                 }
 
-              if (IS_TRUE_SYMOP (IC_JTCOND (ic)) &&
-                  OP_REQV (IC_JTCOND (ic)))
+              if (IS_TRUE_SYMOP (IC_JTCOND (ic)) && OP_REQV (IC_JTCOND (ic)))
                 IC_JTCOND (ic) = opFromOpWithDU (OP_REQV (IC_JTCOND (ic)),
-                                           OP_SYMBOL (IC_JTCOND (ic))->defs,
-                                          OP_SYMBOL (IC_JTCOND (ic))->uses);
+                                                 OP_DEFS (IC_JTCOND (ic)),
+                                                 OP_USES (IC_JTCOND (ic)));
               continue;
             }
 
@@ -1011,14 +1267,16 @@ replaceRegEqv (ebbIndex * ebbi)
               if (POINTER_SET (ic))
                 {
                   IC_RESULT (ic) = opFromOpWithDU (OP_REQV (IC_RESULT (ic)),
-                                           OP_SYMBOL (IC_RESULT (ic))->defs,
-                                          OP_SYMBOL (IC_RESULT (ic))->uses);
+                                                   OP_DEFS (IC_RESULT (ic)),
+                                                   OP_USES (IC_RESULT (ic)));
                   IC_RESULT (ic)->isaddr = 1;
                 }
               else
-                IC_RESULT (ic) = opFromOpWithDU (OP_REQV (IC_RESULT (ic)),
-                                           OP_SYMBOL (IC_RESULT (ic))->defs,
-                                          OP_SYMBOL (IC_RESULT (ic))->uses);
+                {
+                  IC_RESULT (ic) = opFromOpWithDU (OP_REQV (IC_RESULT (ic)),
+                                                   OP_DEFS (IC_RESULT (ic)),
+                                                   OP_USES (IC_RESULT (ic)));
+                }
             }
 
           if (IC_RIGHT (ic) &&
@@ -1037,8 +1295,8 @@ replaceRegEqv (ebbIndex * ebbi)
               OP_REQV (IC_RIGHT (ic)))
             {
               IC_RIGHT (ic) = opFromOpWithDU (OP_REQV (IC_RIGHT (ic)),
-                                            OP_SYMBOL (IC_RIGHT (ic))->defs,
-                                           OP_SYMBOL (IC_RIGHT (ic))->uses);
+                                              OP_DEFS (IC_RIGHT (ic)),
+                                              OP_USES (IC_RIGHT (ic)));
               IC_RIGHT (ic)->isaddr = 0;
             }
 
@@ -1058,8 +1316,8 @@ replaceRegEqv (ebbIndex * ebbi)
               OP_REQV (IC_LEFT (ic)))
             {
               IC_LEFT (ic) = opFromOpWithDU (OP_REQV (IC_LEFT (ic)),
-                                             OP_SYMBOL (IC_LEFT (ic))->defs,
-                                             OP_SYMBOL (IC_LEFT (ic))->uses);
+                                             OP_DEFS (IC_LEFT (ic)),
+                                             OP_USES (IC_LEFT (ic)));
               IC_LEFT (ic)->isaddr = 0;
             }
         }
@@ -1123,7 +1381,6 @@ killDeadCode (ebbIndex * ebbi)
   int gchange = 0;
   int i = 0;
 
-
   /* basic algorithm :-                                          */
   /* first the exclusion rules :-                                */
   /*  1. if result is a global or volatile then skip             */
@@ -1137,9 +1394,8 @@ killDeadCode (ebbIndex * ebbi)
   /*     or then skip                                            */
   /*     else            KILL                                    */
   /* this whole process is carried on iteratively till no change */
-  while (1)
+  do
     {
-
       change = 0;
       /* for all blocks do */
       for (i = 0; i < count; i++)
@@ -1205,9 +1461,7 @@ killDeadCode (ebbIndex * ebbi)
                   for (j = 0; j < count; ebbs[j++]->visited = 0);
 
                   /* find out if this definition is alive */
-                  if (applyToSet (ebbs[i]->succList,
-                                  isDefAlive,
-                                  ic))
+                  if (applyToSet (ebbs[i]->succList, isDefAlive, ic))
                     continue;
 
                   kill = 1;
@@ -1308,17 +1562,13 @@ killDeadCode (ebbIndex * ebbi)
                         bitVectUnSetBit (OP_USES (IC_RIGHT (ic)), ic->key);
                     }
                 }
-
             }                   /* end of all instructions */
 
           if (!ebbs[i]->sch && !ebbs[i]->noPath)
             disconBBlock (ebbs[i], ebbi);
-
         }                       /* end of for all blocks */
-
-      if (!change)
-        break;
-    }                           /* end of while(1) */
+    }                           /* end of do */
+  while (change);
 
   return gchange;
 }
@@ -1394,7 +1644,6 @@ optimizeCastCast (eBBlock ** ebbs, int count)
     {
       for (ic = ebbs[i]->sch; ic; ic = ic->next)
         {
-
           if (ic->op == CAST && IC_RESULT (ic) && IS_ITEMP (IC_RESULT (ic)))
             {
               type1 = operandType (IC_RIGHT (ic));
@@ -1430,6 +1679,10 @@ optimizeCastCast (eBBlock ** ebbs, int count)
               if (SPEC_USIGN (type3) != SPEC_USIGN (type1))
                  continue;
 
+              /* Cast to bool must be preserved to ensure that all nonzero values are correctly cast to true */
+              if (SPEC_NOUN (type2) == V_BOOL && SPEC_NOUN(type3) != V_BOOL)
+                 continue;
+
               /* Change the first cast to a simple assignment and */
               /* let the second cast do all the work */
               ic->op = '=';
@@ -1497,6 +1750,11 @@ eBBlockFromiCode (iCode * ic)
 
   optimizeCastCast (ebbi->bbOrder, ebbi->count);
 
+  /* Burn the corpses, so the dead may rest in peace,
+     safe from cse necromancy */
+  computeDataFlow (ebbi);
+  killDeadCode (ebbi);
+
   /* do common subexpression elimination for each block */
   change = cseAllBlocks (ebbi, FALSE);
 
@@ -1556,31 +1814,33 @@ eBBlockFromiCode (iCode * ic)
 
       if (options.dump_loop)
         dumpEbbsToFileExt (DUMP_LOOPD, ebbi);
-
     }
 
   /* sort it back by block number */
   //qsort (ebbs, saveCount, sizeof (eBBlock *), bbNumCompare);
 
-  if (!options.lessPedantic) {
-    // this is a good place to check missing return values
-    if (currFunc) {
-      // the user is on his own with naked functions...
-      if (!IS_VOID(currFunc->etype)
-       && !FUNC_ISNAKED(currFunc->type)) {
-        eBBlock *bp;
-        // make sure all predecessors of the last block end in a return
-        for (bp=setFirstItem(ebbi->bbOrder[ebbi->count-1]->predList);
-             bp;
-             bp=setNextItem(ebbi->bbOrder[ebbi->count-1]->predList)) {
-          if (bp->ech->op != RETURN) {
-            werrorfl (bp->ech->filename, bp->ech->lineno,
-                      W_VOID_FUNC, currFunc->name);
-          }
+  if (!options.lessPedantic)
+    {
+      // this is a good place to check missing return values
+      if (currFunc)
+        {
+          // the user is on his own with naked functions...
+          if (!IS_VOID(currFunc->etype) && !FUNC_ISNAKED(currFunc->type))
+            {
+              eBBlock *bp;
+              // make sure all predecessors of the last block end in a return
+              for (bp=setFirstItem(ebbi->bbOrder[ebbi->count-1]->predList);
+                   bp;
+                   bp=setNextItem(ebbi->bbOrder[ebbi->count-1]->predList))
+                {
+                  if (bp->ech->op != RETURN)
+                    {
+                      werrorfl (bp->ech->filename, bp->ech->lineno, W_VOID_FUNC, currFunc->name);
+                    }
+                }
+            }
         }
-      }
     }
-  }
 
   /* if cyclomatic info requested then print it */
   if (options.cyclomatic)
@@ -1592,6 +1852,9 @@ eBBlockFromiCode (iCode * ic)
      operations to be as they are for optimizations */
   convertToFcall (ebbi->bbOrder, ebbi->count);
 
+  /* miscelaneous optimizations */
+  miscOpt (ebbi->bbOrder, ebbi->count);
+
   /* compute the live ranges */
   computeLiveRanges (ebbi->bbOrder, ebbi->count, TRUE);
 
@@ -1611,6 +1874,3 @@ eBBlockFromiCode (iCode * ic)
 
   return NULL;
 }
-
-
-/* (add-hook 'c-mode-hook (lambda () (setq c-basic-offset 4))) */
