@@ -317,23 +317,10 @@ void pic16_DumpAop(char *prefix, asmop *aop)
 		DEBUGpic16_emitcode (";", " %s type:AOP_DIR",prefix);
 		DEBUGpic16_emitcode (";", " %s aopu.aop_dir:%s",prefix,aop->aopu.aop_dir);
 	}
-	if (aop->type == AOP_DPTR)
-		DEBUGpic16_emitcode (";", " %s type:AOP_DPTR",prefix);
-	if (aop->type == AOP_DPTR2)
-		DEBUGpic16_emitcode (";", " %s type:AOP_DPTR2",prefix);
-	if (aop->type == AOP_R0)
-		DEBUGpic16_emitcode (";", " %s type:AOP_R0",prefix);
-	if (aop->type == AOP_R1)
-		DEBUGpic16_emitcode (";", " %s type:AOP_R1",prefix);
 	if (aop->type == AOP_STK)
 		DEBUGpic16_emitcode (";", " %s type:AOP_STK",prefix);
 	if (aop->type == AOP_STA)
 		DEBUGpic16_emitcode (";", " %s type:AOP_STA",prefix);
-	if (aop->type == AOP_IMMD)
-	{
-		DEBUGpic16_emitcode (";", " %s type:AOP_IMMD",prefix);
-		DEBUGpic16_emitcode (";", " %s aopu.aop_immd:%s",prefix,aop->aopu.aop_immd);
-	}
 	if (aop->type == AOP_STR)
 	{
 		DEBUGpic16_emitcode (";", " %s type:AOP_STR",prefix);
@@ -443,9 +430,8 @@ void pic16_DumpOp(char *prefix, operand *op)
 	DEBUGpic16_emitcode (";", " %s key:%d",prefix,op->key);
 	if(IS_SYMOP(op)) {
 		sprintf(s,"%s (symOperand)",prefix);
-		pic16_DumpSymbol(s,op->operand.symOperand);
+		pic16_DumpSymbol(s, OP_SYMBOL (op));
 	}
-
 }
 
 void pic16_DumpOpX(FILE *fp, char *prefix, operand *op)
@@ -604,7 +590,7 @@ int pic16_genCmp_special(operand *left, operand *right, operand *result,
     
     if(isAOP_REGlike(left) && isAOP_LIT(right)) {
       /* comparing register vs. literal */
-      lit = (unsigned long)floatFromVal(AOP(right)->aopu.aop_lit);
+      lit = ulFromVal(AOP(right)->aopu.aop_lit);
       
       
       if(size == 1) {
@@ -620,7 +606,7 @@ int pic16_genCmp_special(operand *left, operand *right, operand *result,
                 tmplbl = newiTempLabel( NULL );
                 pic16_emitpcode(POC_TSTFSZ, pic16_popGet(AOP(left), 0));
                 pic16_emitpcode(POC_BRA, pic16_popGetLabel(tmplbl->key));
-                pic16_emitpcode(POC_BRA, pic16_popGetLabel(rIfx->lbl->key));
+                pic16_emitpcode(POC_GOTO, pic16_popGetLabel(rIfx->lbl->key));
                 pic16_emitpLabel(tmplbl->key);
 
                 ifx->generated = 1;
