@@ -25,6 +25,7 @@
 -------------------------------------------------------------------------*/
 
 #include "common.h"
+#include "SDCCsystem.h"
 #include "main.h"
 #include "ralloc.h"
 #include "device.h"
@@ -51,7 +52,7 @@ static char *_pic16_keywords[] =
   "far",
   "interrupt",
   "near",
-  "pdata",
+  //"pdata",
   "reentrant",
   "sfr",
   "sfr16",
@@ -60,7 +61,7 @@ static char *_pic16_keywords[] =
   "_code",
   "_generic",
   "_near",
-  "_pdata",
+  //"_pdata",
   "_naked",
   "shadowregs",
   "wparam",
@@ -81,12 +82,9 @@ static char *_pic16_keywords[] =
 
 pic16_sectioninfo_t pic16_sectioninfo;
 
-int xinst=0;
-
-
 extern char *pic16_processor_base_name(void);
 
-void  pic16_pCodeInitRegisters(void);
+void pic16_pCodeInitRegisters(void);
 
 void pic16_assignRegisters (ebbIndex *);
 
@@ -100,11 +98,10 @@ extern set *libFilesSet;
 
 /* Also defined in gen.h, but the #include is commented out */
 /* for an unknowned reason. - EEP */
-void pic16_emitDebuggerSymbol (char *);
-
-extern void pic16_emitConfigRegs(FILE *of);
-extern void pic16_emitIDRegs(FILE *of);
+void pic16_emitDebuggerSymbol (const char *);
 
+extern void pic16_emitConfigRegs (FILE *of);
+extern void pic16_emitIDRegs (FILE *of);
 
 
 static void
@@ -140,11 +137,11 @@ _pic16_regparm (sym_link * l, bool reentrant)
 
 
 int initsfpnt=0;        /* set to 1 if source provides a pragma for stack
-                 * so glue() later emits code to initialize stack/frame pointers */
+                         * so glue() later emits code to initialize stack/frame pointers */
 set *absSymSet;
 
-set *sectNames=NULL;            /* list of section listed in pragma directives */
-set *sectSyms=NULL;         /* list of symbols set in a specific section */
+set *sectNames=NULL;    /* list of section listed in pragma directives */
+set *sectSyms=NULL;     /* list of symbols set in a specific section */
 set *wparamList=NULL;
 
 #if 0
@@ -163,15 +160,14 @@ struct {
 
 
 enum {
-  P_MAXRAM = 1,
-  P_STACK,
+  P_STACK = 1,
   P_CODE,
   P_UDATA,
   P_LIBRARY
 };
 
 static int
-do_pragma(int id, const char *name, const char *cp)
+do_pragma (int id, const char *name, const char *cp)
 {
   struct pragma_token_s token;
   int err = 0;
@@ -181,36 +177,11 @@ do_pragma(int id, const char *name, const char *cp)
 
   switch (id)
     {
-    /* #pragma maxram [maxram] */
-    case P_MAXRAM:
-      {
-        int max_ram;
-
-        cp = get_pragma_token(cp, &token);
-        if (TOKEN_INT == token.type)
-          max_ram = token.val.int_val;
-        else
-          {
-            err = 1;
-            break;
-          }
-
-        cp = get_pragma_token(cp, &token);
-        if (TOKEN_EOL != token.type)
-          {
-            err = 1;
-            break;
-          }
-
-        pic16_setMaxRAM(max_ram);
-      }
-      break;
-
     /* #pragma stack [stack-position] [stack-len] */
     case  P_STACK:
       {
         unsigned int stackPos, stackLen;
-        regs *reg;
+        reg_info *reg;
         symbol *sym;
 
         cp = get_pragma_token(cp, &token);
@@ -502,7 +473,6 @@ do_pragma(int id, const char *name, const char *cp)
 }
 
 static struct pragma_s pragma_tbl[] = {
-  { "maxram",  P_MAXRAM,  0, do_pragma },
   { "stack",   P_STACK,   0, do_pragma },
   { "code",    P_CODE,    0, do_pragma },
   { "udata",   P_UDATA,   0, do_pragma },
@@ -512,53 +482,53 @@ static struct pragma_s pragma_tbl[] = {
   };
 
 static int
-_process_pragma(const char *s)
+_process_pragma (const char *s)
 {
   return process_pragma_tbl(pragma_tbl, s);
 }
 
-#define REP_UDATA       "--preplace-udata-with="
+#define REP_UDATA         "--preplace-udata-with="
 
-#define STACK_MODEL     "--pstack-model="
-#define OPT_BANKSEL     "--obanksel="
+#define STACK_MODEL       "--pstack-model="
+#define OPT_BANKSEL       "--obanksel="
 
-#define ALT_ASM         "--asm="
-#define ALT_LINK        "--link="
+#define ALT_ASM           "--asm="
+#define ALT_LINK          "--link="
 
-#define IVT_LOC         "--ivt-loc="
-#define NO_DEFLIBS      "--nodefaultlibs"
-#define MPLAB_COMPAT    "--mplab-comp"
+#define IVT_LOC           "--ivt-loc="
+#define NO_DEFLIBS        "--nodefaultlibs"
+#define MPLAB_COMPAT      "--mplab-comp"
 
-#define USE_CRT         "--use-crt="
+#define USE_CRT           "--use-crt="
 
-#define OFMSG_LRSUPPORT "--flr-support"
+#define OFMSG_LRSUPPORT   "--flr-support"
 
-#define OPTIMIZE_GOTO   "--optimize-goto"
-#define OPTIMIZE_CMP    "--optimize-cmp"
-#define OPTIMIZE_DF     "--optimize-df"
+#define NO_OPTIMIZE_GOTO  "--no-optimize-goto"
+#define OPTIMIZE_CMP      "--optimize-cmp"
+#define OPTIMIZE_DF       "--optimize-df"
 
-char *alt_asm=NULL;
-char *alt_link=NULL;
+char *alt_asm = NULL;
+char *alt_link = NULL;
 
-int pic16_mplab_comp=0;
+int pic16_mplab_comp = 0;
 extern int pic16_debug_verbose;
 extern int pic16_ralloc_debug;
 extern int pic16_pcode_verbose;
 
-int pic16_enable_peeps=0;
+int pic16_enable_peeps = 0;
 
 OPTION pic16_optionsTable[]= {
     /* code generation options */
     { 0, STACK_MODEL,        NULL, "use stack model 'small' (default) or 'large'"},
 #if XINST
-    { 'y', "--extended",     &xinst, "enable Extended Instruction Set/Literal Offset Addressing mode"},
+    { 'y', "--extended",     &pic16_options.xinst, "enable Extended Instruction Set/Literal Offset Addressing mode"},
 #endif
     { 0, "--pno-banksel",    &pic16_options.no_banksel, "do not generate BANKSEL assembler directives"},
 
     /* optimization options */
     { 0, OPT_BANKSEL,       &pic16_options.opt_banksel, "set banksel optimization level (default=0 no)", CLAT_INTEGER },
     { 0, "--denable-peeps", &pic16_enable_peeps, "explicit enable of peepholes"},
-    { 0, OPTIMIZE_GOTO,     NULL, "try to use (conditional) BRA instead of GOTO"},
+    { 0, NO_OPTIMIZE_GOTO,  NULL, "do NOT use (conditional) BRA instead of GOTO"},
     { 0, OPTIMIZE_CMP,      NULL, "try to optimize some compares"},
     { 0, OPTIMIZE_DF,       NULL, "thoroughly analyze data flow (memory and time intensive!)"},
 
@@ -636,8 +606,8 @@ _pic16_parseOptions (int *pargc, char **argv, int *i)
     }
 #endif
 
-    if (ISOPT(OPTIMIZE_GOTO)) {
-      pic16_options.opt_flags |= OF_OPTIMIZE_GOTO;
+    if (ISOPT(NO_OPTIMIZE_GOTO)) {
+      pic16_options.opt_flags |= OF_NO_OPTIMIZE_GOTO;
       return TRUE;
     }
 
@@ -655,61 +625,16 @@ _pic16_parseOptions (int *pargc, char **argv, int *i)
   return FALSE;
 }
 
-extern set *userIncDirsSet;
+extern void pic16_init_pic(const char *name);
 
-static void _pic16_initPaths(void)
+static void
+_pic16_initPaths (void)
 {
-  set *pic16incDirsSet=NULL;
-  set *pic16libDirsSet=NULL;
-  char devlib[512];
-
-    setMainValue("mcu", pic16->name[2] );
-    addSet(&preArgvSet, Safe_strdup("-D{mcu}"));
-
-    setMainValue("mcu1", pic16->name[1] );
-    addSet(&preArgvSet, Safe_strdup("-D__{mcu1}"));
-
-    if(!options.nostdinc) {
-      struct dbuf_s pic16incDir;
-
-      dbuf_init(&pic16incDir, 128);
-      dbuf_makePath(&pic16incDir, INCLUDE_DIR_SUFFIX, "pic16");
-
-      /* setup pic16 include directory */
-      pic16incDirsSet = appendStrSet(dataDirsSet, NULL, dbuf_c_str(&pic16incDir));
-      dbuf_destroy(&pic16incDir);
-      includeDirsSet = pic16incDirsSet;
-//      mergeSets(&includeDirsSet, pic16incDirsSet);
-    }
-    /* pic16 port should not search to the SDCC standard include directories,
-     * so add here the deleted include dirs that user has issued in command line */
-    mergeSets(&pic16incDirsSet, userIncDirsSet);
-
-    if(!options.nostdlib) {
-      struct dbuf_s pic16libDir;
-
-      dbuf_init(&pic16libDir, 128);
-      dbuf_makePath(&pic16libDir, LIB_DIR_SUFFIX, "pic16");
-      /* setup pic16 library directory */
-      pic16libDirsSet = appendStrSet(dataDirsSet, NULL, dbuf_c_str(&pic16libDir));
-      dbuf_destroy(&pic16libDir);
-      libDirsSet = pic16libDirsSet;
-//      mergeSets(&libDirsSet, pic16libDirsSet);
-    }
-
-    if(!pic16_options.nodefaultlibs) {
-      /* now add the library for the device */
-      sprintf(devlib, "%s.lib", pic16->name[2]);
-      addSet(&libFilesSet, Safe_strdup(devlib));
-
-      /* add the internal SDCC library */
-      addSet(&libFilesSet, Safe_strdup( "libsdcc.lib" ));
-    }
+  pic16_init_pic(port->processor);
 }
 
 extern set *linkOptionsSet;
 char *msprintf(hTab *pvals, const char *pformat, ...);
-int my_system(const char *cmd);
 
 /* forward declarations */
 extern const char *pic16_linkCmd[];
@@ -717,161 +642,158 @@ extern const char *pic16_asmCmd[];
 extern set *asmOptionsSet;
 
 /* custom function to link objects */
-static void _pic16_linkEdit(void)
+static void
+_pic16_linkEdit (void)
 {
-  hTab *linkValues=NULL;
+  hTab *linkValues = NULL;
   char lfrm[1024];
   char *lcmd;
-  char temp[1024];
-  set *tSet=NULL;
+  char temp[PATH_MAX];
+  set *tSet = NULL;
   int ret;
 
-    /*
-     * link command format:
-     * {linker} {incdirs} {lflags} -o {outfile} {spec_ofiles} {ofiles} {libs}
-     *
-     */
-    sprintf(lfrm, "{linker} {incdirs} {lflags} -w -r -o {outfile} {user_ofile} {ofiles} {spec_ofiles} {libs}");
+  /*
+   * link command format:
+   * {linker} {incdirs} {lflags} -o {outfile} {spec_ofiles} {ofiles} {libs}
+   *
+   */
+  sprintf (lfrm, "{linker} {incdirs} {lflags} -w -r -o \"{outfile}\" \"{user_ofile}\" {ofiles} {spec_ofiles} {libs}");
 
-    shash_add(&linkValues, "linker", pic16_linkCmd[0]);
+  shash_add (&linkValues, "linker", pic16_linkCmd[0]);
 
-    mergeSets(&tSet, libDirsSet);
-    mergeSets(&tSet, libPathsSet);
+  mergeSets (&tSet, libPathsSet);
+  mergeSets (&tSet, libDirsSet);
 
-    shash_add(&linkValues, "incdirs", joinStrSet( appendStrSet(tSet, "-I\"", "\"")));
-    shash_add(&linkValues, "lflags", joinStrSet(linkOptionsSet));
+  shash_add (&linkValues, "incdirs", joinStrSet (appendStrSet(tSet, "-I\"", "\"")));
+  shash_add (&linkValues, "lflags", joinStrSet (linkOptionsSet));
 
-    shash_add(&linkValues, "outfile", fullDstFileName ? fullDstFileName : dstFileName);
+  shash_add (&linkValues, "outfile", fullDstFileName ? fullDstFileName : dstFileName);
 
-    if(fullSrcFileName) {
-        sprintf(temp, "%s.o", fullDstFileName ? fullDstFileName : dstFileName);
-//      addSetHead(&relFilesSet, Safe_strdup(temp));
-                shash_add(&linkValues, "user_ofile", temp);
+  if (fullSrcFileName)
+    {
+      SNPRINTF (temp, sizeof (temp), "%s.o", fullDstFileName ? fullDstFileName : dstFileName);
+//    addSetHead (&relFilesSet, Safe_strdup (temp));
+      shash_add (&linkValues, "user_ofile", temp);
     }
 
-    if(!pic16_options.no_crt)
-          shash_add(&linkValues, "spec_ofiles", pic16_options.crt_name);
+  if (!pic16_options.no_crt)
+    shash_add (&linkValues, "spec_ofiles", pic16_options.crt_name);
 
-    shash_add(&linkValues, "ofiles", joinStrSet(relFilesSet));
+  shash_add (&linkValues, "ofiles", joinStrSet (appendStrSet (relFilesSet, "\"", "\"")));
 
-    if(!libflags.ignore) {
-      if(libflags.want_libc)
-        addSet(&libFilesSet, Safe_strdup("libc18f.lib"));
+  if (!libflags.ignore)
+    {
+      if (libflags.want_libc)
+        addSet (&libFilesSet, Safe_strdup ("libc18f.lib"));
 
-          if(libflags.want_libm)
-            addSet(&libFilesSet, Safe_strdup("libm18f.lib"));
+      if (libflags.want_libm)
+        addSet (&libFilesSet, Safe_strdup ("libm18f.lib"));
 
-          if(libflags.want_libio) {
-            sprintf(temp, "libio%s.lib", pic16->name[1]);   /* build libio18f452.lib name */
-            addSet(&libFilesSet, Safe_strdup(temp));
-          }
-
-          if(libflags.want_libdebug)
-            addSet(&libFilesSet, Safe_strdup("libdebug.lib"));
+      if (libflags.want_libio)
+        {
+          SNPRINTF (temp, sizeof (temp), "libio%s.lib", pic16->name[1]);   /* build libio18f452.lib name */
+          addSet (&libFilesSet, Safe_strdup (temp));
         }
 
-    shash_add(&linkValues, "libs", joinStrSet(libFilesSet));
+      if (libflags.want_libdebug)
+        addSet(&libFilesSet, Safe_strdup ("libdebug.lib"));
+    }
 
-    lcmd = msprintf(linkValues, lfrm);
+  shash_add (&linkValues, "libs", joinStrSet (appendStrSet (libFilesSet, "\"", "\"")));
 
-    ret = my_system( lcmd );
+  lcmd = msprintf(linkValues, lfrm);
 
-    Safe_free( lcmd );
+  ret = sdcc_system (lcmd);
 
-    if(ret)
-        exit(1);
+  Safe_free (lcmd);
+
+  if (ret)
+      exit (1);
 }
 
 
 static void
 _pic16_finaliseOptions (void)
 {
-    port->mem.default_local_map = data;
-    port->mem.default_globl_map = data;
+  port->mem.default_local_map = data;
+  port->mem.default_globl_map = data;
 
-    /* peepholes are disabled for the time being */
-    options.nopeep = 1;
+  /* peepholes are disabled for the time being */
+  options.nopeep = 1;
 
-    /* explicit enable peepholes for testing */
-    if(pic16_enable_peeps)
-      options.nopeep = 0;
+  /* explicit enable peepholes for testing */
+  if (pic16_enable_peeps)
+    options.nopeep = 0;
 
-    options.all_callee_saves = 1;       // always callee saves
+  options.all_callee_saves = 1;       // always callee saves
 
 #if 0
-    options.float_rent = 1;
-    options.intlong_rent = 1;
+  options.float_rent = 1;
+  options.intlong_rent = 1;
 #endif
 
+  setMainValue("mcu", pic16->name[2] );
+  addSet(&preArgvSet, Safe_strdup("-D{mcu}"));
 
-    if(alt_asm && strlen(alt_asm))
+  setMainValue("mcu1", pic16->name[1] );
+  addSet(&preArgvSet, Safe_strdup("-D__{mcu1}"));
+
+  if (!pic16_options.nodefaultlibs)
+    {
+      char devlib[512];
+
+      /* now add the library for the device */
+      sprintf(devlib, "libdev%s.lib", pic16->name[1]);   /* e.g., libdev18f452.lib */
+      addSet(&libFilesSet, Safe_strdup(devlib));
+
+      /* add the internal SDCC library */
+      addSet(&libFilesSet, Safe_strdup( "libsdcc.lib" ));
+    }
+
+  if (alt_asm && alt_asm[0] != '\0')
+    {
       pic16_asmCmd[0] = alt_asm;
+    }
 
-    if(alt_link && strlen(alt_link))
+  if (alt_link && alt_link[0] != '\0')
+    {
       pic16_linkCmd[0] = alt_link;
+    }
 
-    if(!pic16_options.no_crt) {
+  if (!pic16_options.no_crt)
+    {
       pic16_options.omit_ivt = 1;
       pic16_options.leave_reset = 0;
     }
 
-    if(options.model == MODEL_SMALL)
+  if (options.model == MODEL_SMALL)
+    {
       addSet(&asmOptionsSet, Safe_strdup("-DSDCC_MODEL_SMALL"));
-    else
-    if(options.model == MODEL_LARGE)
-      addSet(&asmOptionsSet, Safe_strdup("-DSDCC_MODEL_LARGE"));
-
+    }
+  else if (options.model == MODEL_LARGE)
     {
       char buf[128];
 
-        sprintf(buf, "-D%s -D__%s", pic16->name[2], pic16->name[1]);
-        *(strrchr(buf, 'f')) = 'F';
-        addSet(&asmOptionsSet, Safe_strdup( buf ));
+      addSet(&asmOptionsSet, Safe_strdup("-DSDCC_MODEL_LARGE"));
+
+      sprintf(buf, "-D%s -D__%s", pic16->name[2], pic16->name[1]);
+      *(strrchr(buf, 'f')) = 'F';
+      addSet(&asmOptionsSet, Safe_strdup(buf));
     }
 
-    if(STACK_MODEL_LARGE) {
+  if (STACK_MODEL_LARGE)
+    {
       addSet(&preArgvSet, Safe_strdup("-DSTACK_MODEL_LARGE"));
       addSet(&asmOptionsSet, Safe_strdup("-DSTACK_MODEL_LARGE"));
-    } else {
+    }
+  else
+    {
       addSet(&preArgvSet, Safe_strdup("-DSTACK_MODEL_SMALL"));
       addSet(&asmOptionsSet, Safe_strdup("-DSTACK_MODEL_SMALL"));
     }
 }
 
 
-#if 0
-  if (options.model == MODEL_LARGE)
-    {
-      port->mem.default_local_map = xdata;
-      port->mem.default_globl_map = xdata;
-    }
-  else
-    {
-      port->mem.default_local_map = data;
-      port->mem.default_globl_map = data;
-    }
-
-  if (options.stack10bit)
-    {
-      if (options.model != MODEL_FLAT24)
-    {
-      fprintf (stderr,
-           "*** warning: 10 bit stack mode is only supported in flat24 model.\n");
-      fprintf (stderr, "\t10 bit stack mode disabled.\n");
-      options.stack10bit = 0;
-    }
-      else
-    {
-      /* Fixup the memory map for the stack; it is now in
-       * far space and requires a FPOINTER to access it.
-       */
-      istack->fmap = 1;
-      istack->ptrType = FPOINTER;
-    }
-    }
-#endif
-
-
 static void
 _pic16_setDefaultOptions (void)
 {
@@ -900,7 +822,7 @@ _pic16_setDefaultOptions (void)
 }
 
 static const char *
-_pic16_getRegName (struct regs *reg)
+_pic16_getRegName (const struct reg_info *reg)
 {
   if (reg)
     return reg->name;
@@ -909,7 +831,8 @@ _pic16_getRegName (struct regs *reg)
 
 
 #if 1
-static  char *_pic16_mangleFunctionName(char *sz)
+static const char *
+_pic16_mangleFunctionName (const char *sz)
 {
 //  fprintf(stderr, "mangled function name: %s\n", sz);
 
@@ -982,7 +905,8 @@ _pic16_genIVT (struct dbuf_s * oBuf, symbol ** interrupts, int maxInterrupts)
 
 /* return True if the port can handle the type,
  * False to convert it to function call */
-static bool _hasNativeMulFor (iCode *ic, sym_link *left, sym_link *right)
+static bool
+_hasNativeMulFor (iCode *ic, sym_link *left, sym_link *right)
 {
   //fprintf(stderr,"checking for native mult for %c (size: %d)\n", ic->op, getSize(OP_SYMBOL(IC_RESULT(ic))->type));
   int symL, symR, symRes, sizeL = 0, sizeR = 0, sizeRes = 0;
@@ -1172,8 +1096,6 @@ const char *pic16_linkCmd[] =
   "gplink", "$l", "-w", "-r", "-o \"$2\"", "\"$1\"","$3", NULL
 };
 
-
-
 /** $1 is always the basename.
     $2 is always the output file.
     $3 varies (nothing currently)
@@ -1182,8 +1104,7 @@ const char *pic16_linkCmd[] =
 */
 const char *pic16_asmCmd[] =
 {
-  "gpasm", "$l", "$3", "-c", "\"$1.asm\"", "-o \"$2\"", NULL
-
+  "gpasm", "$l", "$3", "-o", "\"$2\"", "-c", "\"$1.asm\"", NULL
 };
 
 /* Globals */
@@ -1196,8 +1117,9 @@ PORT pic16_port =
   {
     pic16glue,
     TRUE,           /* Emit glue around main */
-    MODEL_SMALL | MODEL_LARGE | MODEL_FLAT24,
-    MODEL_SMALL
+    NO_MODEL,
+    NO_MODEL,
+    NULL,           /* model == target */
   },
   {
     pic16_asmCmd,   /* assembler command and arguments */
@@ -1215,7 +1137,7 @@ PORT pic16_port =
     ".o",           /* extension for object files */
     0               /* no need for linker file */
   },
-  {
+  {                 /* Peephole optimizer */
     _defaultRules
   },
   {
@@ -1224,6 +1146,7 @@ PORT pic16_port =
     2,      /* short */
     2,      /* int */
     4,      /* long */
+    8,      /* long long */
     2,      /* ptr */
     3,      /* fptr, far pointers (see Microchip) */
     3,      /* gptr */
